'----------------------------- GUI.BAS -------------------------------
'
' This module is a collection of routines to emulate the MS
' Windows 3.x GUI.
'
'-----------------------------------------------------------------------
DEFINT A-Z
     '--- Object Type Structure

     TYPE GUIObject
        CurrentUse          AS INTEGER ' Current Use of this Object
        ULC                 AS INTEGER ' Upper Left Column
        ULR                 AS INTEGER ' Upper Left Row
        LRC                 AS INTEGER ' Lower Right Column 
        LRR                 AS INTEGER ' Lower Right Row 
        ObjectType          AS INTEGER ' Object Type
        State               AS INTEGER ' Current State of the Object
        Visible             AS INTEGER ' Visible Parameter
        Enabled             AS INTEGER ' Enabled Parameter
        Fore                AS INTEGER ' Foreground Color
        Back                AS INTEGER ' Background Color
        HotKeyColor         AS INTEGER ' Text Box Hot Key Color
        BarPosition         AS INTEGER ' Current Vert/Horz Bar Position
        DirectoryType       AS INTEGER ' Directory type for Directory Icon
        LongestStringLen    AS INTEGER ' Combo Box parameter
        TextJustify         AS INTEGER ' Text Box parameter 
        TextStrLen          AS INTEGER ' Text String Len
        Text                AS STRING * 96  ' Text associated with some Objects
     END TYPE            
     
     '---  Constants  
     CONST TRUE = -1
     CONST FALSE = 0
     
     '---  State Constants  
     CONST DEPRESSED = 1
     CONST FOCUS = 2
     CONST SELECTED = 4
     CONST UPDATEONLY = 8
     CONST INAVLIDSTATE = &HFF00
     
     '---  Mouse Button Constants  
     CONST MOUSEBUTTONDEPRESSED = 1
     CONST MOUSEBUTTONCLICK = 2
     
     '---  Vertical Scroll Bar Constants  
     CONST TOPDEPRESSED = 16
     CONST BOTTOMDEPRESSED = 32
     
     '---  Horizontal Scroll Bar Constants  
     CONST LEFTDEPRESSED = 64
     CONST RIGHTDEPRESSED = 128
     
     '---  Text Box Justify Constants  
     CONST LEFTJUSTIFY = 0
     CONST CENTERJUSTIFY = 1
     CONST RIGHTJUSTIFY = 2
     
     '---  Color Constants  
     CONST BLACK = 0
     CONST BLUE = 1
     CONST GREEN = 2
     CONST CYAN = 3
     CONST RED = 4
     CONST MAGENTA = 5
     CONST BROWN = 6
     CONST LIGHTGRAY = 7
     CONST DARKGRAY = 8
     CONST LIGHTBLUE = 9
     CONST LIGHTGREEN = 10
     CONST LIGHTCYAN = 11
     CONST LIGHTRED = 12
     CONST LIGHTMAGENTA = 13
     CONST YELLOW = 14
     CONST WHITE = 15
     
     '---  Object Type Constants  
     CONST TextBoxType             = 1
     CONST FileIconType            = 2
     CONST ExeIconType             = 3
     CONST DirIconType             = 4
     CONST ComboBoxType            = 5
     CONST RightArrowButType       = 6
     CONST LeftArrowButType        = 7
     CONST DownArrowButType        = 8
     CONST UpArrowButType          = 9
     CONST FloppyDriveType         = 10
     CONST HardDriveType           = 11
     CONST MinMaxButType           = 12
     CONST MaxButType              = 13
     CONST MinButType              = 14
     CONST ControlBoxType          = 15
     CONST HorizontalScrollBarType = 16
     CONST VerticalScrollBarType   = 17
     CONST FrameBoxType            = 18
     CONST OptionButType           = 19
     CONST CheckBoxType            = 20
     CONST CommandButType          = 21
     CONST MaxObjectType           = 21
     
     '---  Drawing Routines  
DECLARE SUB DrawTextBox (ULC%, ULR%, LRC%, LRR%, State%, Justify%, Caption$, HotColor%, Fore%, Back%) '
DECLARE SUB DrawFileIcon (ULC%, ULR%, State%, FileName$, Fore%, Back%) '
DECLARE SUB DrawExeIcon (ULC%, ULR%, State%, FileName$, Fore%, Back%) '
DECLARE SUB DrawDirIcon (ULC%, ULR%, State%, DirType%, DirName$, Fore%, Back%) '
DECLARE SUB DrawComboBox (ULC%, ULR%, State%, LongestStringLen%, Caption$) '
DECLARE SUB DrawRightArrowBut (ULC%, ULR%, State%) '
DECLARE SUB DrawLeftArrowBut (ULC%, ULR%, State%) '
DECLARE SUB DrawDownArrowBut (ULC%, ULR%, State%) '
DECLARE SUB DrawUpArrowBut (ULC%, ULR%, State%) '
DECLARE SUB DrawHardDrive (ULC%, ULR%, State%, DriveLetter$) '
DECLARE SUB DrawFloppyDrive (ULC%, ULR%, State%, DriveLetter$) '
DECLARE SUB DrawMinMaxBut (ULC%, ULR%, State%) '
DECLARE SUB DrawMaxBut (ULC%, ULR%, State%) '
DECLARE SUB DrawMinBut (ULC%, ULR%, State%) '
DECLARE SUB DrawControlBox (ULC%, ULR%, State%) '
DECLARE SUB DrawHorizontalScrollBar (ULC%, ULR%, LRC%, LRR%, State%, BarPosition%) '
DECLARE SUB DrawVerticalScrollBar (ULC%, ULR%, LRC%, LRR%, State%, BarPosition%) '
DECLARE SUB DrawFrameBox (ULC%, ULR%, LRC%, LRR%, FrameTitle$, FrameColor%, FillColor%) '
DECLARE SUB DrawOptionBut (ULC%, ULR%, State%, Caption$, Fore%, Back%) '
DECLARE SUB DrawCheckBox (ULC%, ULR%, State%, Caption$, Fore%, Back%) '
DECLARE SUB DrawCommandBut (ULC%, ULR%, State%, Caption$) '
     
     
     '---  Sizing Routines  
DECLARE SUB SizeTextBox (ULC%, ULR%, LRC%, LRR%, State%, Justify%, Caption$, HotColor%, Fore%, Back%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeFileIcon (ULC%, ULR%, State%, FileName$, Fore%, Back%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeExeIcon (ULC%, ULR%, State%, FileName$, Fore%, Back%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeDirIcon (ULC%, ULR%, State%, DirType%, DirName$, Fore%, Back%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeComboBox (ULC%, ULR%, State%, LongestStringLen%, Caption$, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeRightArrowBut (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeLeftArrowBut (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeDownArrowBut (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeUpArrowBut (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeHardDrive (ULC%, ULR%, State%, DriveLetter$, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeFloppyDrive (ULC%, ULR%, State%, DriveLetter$, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeMinMaxBut (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeMaxBut (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeMinBut (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeControlBox (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeHorizontalScrollBar (ULC%, ULR%, LRC%, LRR%, State%, BarPosition%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeVerticalScrollBar (ULC%, ULR%, LRC%, LRR%, State%, BarPosition%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeOptionBut (ULC%, ULR%, State%, Caption$, Fore%, Back%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeCheckBox (ULC%, ULR%, State%, Caption$, Fore%, Back%, ObjectHeight%, ObjectWidth%) '
DECLARE SUB SizeCommandBut (ULC%, ULR%, State%, Caption$, ObjectHeight%, ObjectWidth%) '
     
     
     '---  Sizing Constants  
     CONST FileIconHeight = 18
     CONST FileIconWidth = 147
     CONST DirIconHeight = 18
     CONST DirIconWidth = 147
     CONST ExeIconHeight = 18
     CONST ExeIconWidth = 147
     CONST HardDriveHeight = 27
     CONST HardDriveWidth =  69
     CONST FloppyDriveHeight = 27
     CONST FloppyDriveWidth =  69
     CONST HorizontalScrollBarHeight = 21
     CONST VerticalScrollBarWidth = 21
     
     CONST ScrollBarBoxSize = 21
     
     '---  Setting Routines  
DECLARE FUNCTION SetTextBox% (ULC%, ULR%, LRC%, LRR%, State%, Justify%, Caption$, HotColor%, Fore%, Back%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetFileIcon% (ULC%, ULR%, State%, FileName$, Fore%, Back%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetExeIcon% (ULC%, ULR%, State%, FileName$, Fore%, Back%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetDirIcon% (ULC%, ULR%, State%, DirType%, DirName$, Fore%, Back%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetComboBox% (ULC%, ULR%, State%, LongestStringLen%, Caption$, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetRightArrowBut% (ULC%, ULR%, State%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetLeftArrowBut% (ULC%, ULR%, State%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetDownArrowBut% (ULC%, ULR%, State%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetUpArrowBut% (ULC%, ULR%, State%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetHardDrive% (ULC%, ULR%, State%, DriveLetter$, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetFloppyDrive% (ULC%, ULR%, State%, DriveLetter$, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetMinMaxBut% (ULC%, ULR%, State%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetMaxBut% (ULC%, ULR%, State%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetMinBut% (ULC%, ULR%, State%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetControlBox% (ULC%, ULR%, State%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetHorizontalScrollBar% (ULC%, ULR%, LRC%, LRR%, State%, BarPosition%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetFrameBox% (ULC%, ULR%, LRC%, LRR%, FrameTitle$, FrameColor%, FillColor%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetVerticalScrollBar% (ULC%, ULR%, LRC%, LRR%, State%, BarPosition%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetOptionBut% (ULC%, ULR%, State%, Caption$, Fore%, Back%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetCheckBox% (ULC%, ULR%, State%, Caption$, Fore%, Back%, ObjectArray() AS GUIObject) '
DECLARE FUNCTION SetCommandBut% (ULC%, ULR%, State%, Caption$, ObjectArray() AS GUIObject) '


     '---  Other Subroutines  
DECLARE SUB FlushGUIEvent () '
DECLARE SUB CheckGUIEvent (ObjectHandle%, CMD1%, CMD2%, ObjectArray() AS GUIObject) '
DECLARE SUB SetSemiAutoGUI (SemiAutomaticFlag%) '
DECLARE SUB SetObjectVisible (VisibleFlag%, ObjectHandle%, ObjectArray() AS GUIObject) '
DECLARE SUB SetObjectEnabled (EnabledFlag%, ObjectHandle%, ObjectArray() AS GUIObject) '
DECLARE SUB DeleteObject (ObjectHandle%, ObjectArray() AS GUIObject) '
DECLARE SUB AnimateObject (ObjectHandle%, State%, ObjectArray() AS GUIObject) '
DECLARE SUB AnimateScrollBarObject (ObjectHandle%, State%, BarPosition%, ObjectArray() AS GUIObject) '
DECLARE SUB InitObjectArray (ObjectCount%, ObjectArray() AS GUIObject) '
DECLARE SUB UpdateGUI (ObjectArray() AS GUIObject) '
DECLARE FUNCTION GETCHARSIZE%
DECLARE FUNCTION GetHandle% (TempObjectArray() AS GUIObject)
DECLARE SUB DrawGenericDrive (ULC%, ULR%)
DECLARE SUB DrawGenericButton (ULC%, ULR%, LRC%, LRR%, State%)
DECLARE SUB FocusBox (ULC%, ULR%, LRC%, LRR%, ForeColor%)

     COMMON SHARED /GuiModule/ SemiAutoGUIFlag%

' ---------------------------------------------------------------------------
'   Program: Main
' ---------------------------------------------------------------------------
DECLARE FUNCTION DIRFIRST% (FileSpec$, BYVAL FileAttrib%)
DECLARE FUNCTION DIRNEXT% ()
DECLARE FUNCTION DIRFILEATTRIB% ()      ' file attribute byte
DECLARE FUNCTION DIRFILENAME$ ()        ' file name only
DECLARE FUNCTION DIRFILESIZE& ()        ' file size
DECLARE SUB DIRFILEAGE (Mo%, Da%, Yr%, Hr%, Mn%)
DECLARE SUB DIRFILEINFO (s$)            ' formatted file name
DECLARE SUB DIRGETDTA (BYVAL Segment%, BYVAL Offset%)
DECLARE SUB DIRSETDTA (BYVAL Segment%, BYVAL Offset%)
DECLARE FUNCTION DIRCOUNT% (FileSpec$, BYVAL FileAttrib%)
DECLARE FUNCTION DIRMONTH$ (BYVAL MonthNumber%) ' only if SpellMonth !=0

' --------------------------------------------------------------------------

'--- Do we have a mouse to run the demo with ?

CALL MMCHECK(MousePresent%)
IF MousePresent% = 0 THEN
   PRINT
   PRINT "No mouse installed!  Can't run program."
   PRINT
   END
END IF

' ===========================================================================
'
'  Function:
'
'     PARAMETERS PASSED TO ROUTINE:
'
'                         NONE
'
'     PARAMETERS RETURNED FROM ROUTINE:
'
'                     SetAdapter% - Returns -1 if graphics mode can be set
'                                   for GUI.BAS.  Returns 0 is graphics
'                                   mode is not supported by GUI.BAS and
'                                   also sets the screen to mode 0, 80x25
'                                   text only.
'
'                           XMAX% - Returns pixel limit for graphics screen.
'                           YMAX% -    DITO
'
' =========<EndOfDoc>=======================================================

  ON ERROR GOTO Handler

  '
  ' Use an array to keep track of our adapter test results.
  '
  REDIM mode(1 TO 13)

  '
  ' Try ALL Qb-screen modes.
  '
  FOR ModeNumber = 1 TO 13
      mode(ModeNumber) = ModeNumber ' Assume test works unless you get an error
      SCREEN ModeNumber             ' in which case the the error handler sets
  NEXT ModeNumber                   ' the value of Mode() = 0

  '
  ' Using test results figure out which adapter is active and optimize
  ' the screen for GRAPHICS usage (if available).
  '
  ' (See the tables in SCREEN section of BASIC Language Reference
  ' to undertand how this logic works.)

  IF mode(13) THEN
     IF mode(7) THEN                                    ' VGA adapter
        AdapterType = 1
        GOSUB RecordModes
     ELSE                                               ' MCGA adapter
        AdapterType = 2
        GOSUB RecordModes
     END IF
  ELSE
     IF mode(7) THEN                                    ' EGA adapter
        AdapterType = 3
        GOSUB RecordModes
     ELSE
        IF mode(3) THEN                                 ' HERC adapter
           AdapterType = 4
           GOSUB RecordModes
        END IF
        IF mode(4) THEN                                 ' Olivetti, AT&T
           AdapterType = 5
           GOSUB RecordModes
        END IF
        IF mode(1) THEN                                 ' CGA adapter
           AdapterType = 6
           GOSUB RecordModes
        ELSE                                            ' MDPA adapter
           AdapterType = 7
           UseableModes$ = "0"
        END IF
     END IF
  END IF

  s = 0
  SELECT CASE AdapterType
   CASE 1
    IF INSTR(UseableModes$, " 8 ") THEN s = 8: w = 80: h = 25: XMAX = 640: YMAX = 200
    IF INSTR(UseableModes$, " 9 ") THEN s = 9: w = 80: h = 43: XMAX = 640: YMAX = 350
    IF INSTR(UseableModes$, " 11 ") THEN s = 11: w = 80: h = 60: XMAX = 640: YMAX = 480
    IF INSTR(UseableModes$, " 12 ") THEN s = 12: w = 80: h = 60: XMAX = 640: YMAX = 480
   CASE 3
    IF INSTR(UseableModes$, " 8 ") THEN s = 8: w = 80: h = 25: XMAX = 640: YMAX = 200
    IF INSTR(UseableModes$, " 9 ") THEN s = 9: w = 80: h = 43: XMAX = 640: YMAX = 350
   CASE ELSE
  END SELECT

  ON ERROR GOTO SetScrErr

  '
  ' Set screen to best graphics mode
  '
  IF s THEN
     SCREEN s
     WIDTH w, h
     SetAdapter = -1
  END IF

' -------------  end of routine SetAdapter  ------------------------

IF SetAdapter = 0 THEN
   PRINT
   PRINT "Sorry, but your display adapter will not work with program."
   PRINT
   END
END IF
XMAX = XMAX - 1                                 ' Reduce X/Y by one
YMAX = YMAX - 1

'--- Count the files in the directory

FSpec$ = "*.*"                                  ' set search File Specs
FAttr = 0                                       ' find all normal files
NumberOfFiles = DIRCOUNT(FSpec$, FAttr)         ' get file count

'--- Set limit for the number of files to display based on screen pixle count

IF YMAX = 199 THEN
   MaxFiles = 18
ELSE
   MaxFiles = 60
END IF

IF YMAX = 199 THEN
   IndexLimit = 6
ELSE
   IndexLimit = 20
END IF

IF NumberOfFiles > MaxFiles THEN
   NumberOfFiles = MaxFiles
END IF

IF (NumberOfFiles < IndexLimit) THEN
   MaxIndex = NumberOfFiles
ELSE
   MaxIndex = IndexLimit
END IF

'--- Is the file count odd or even ?

IF NumberOfFiles MOD IndexLimit = 0 THEN
   AddFactor = 0
ELSE
   AddFactor = 1
END IF

'--- Set number of coulmns to display

NumberOfColumns = (NumberOfFiles \ IndexLimit) + AddFactor

' --------------------------------------------------------------------------

NumberOfObjects = 2                               ' set number of objects
DIM Objects2(0 TO NumberOfObjects) AS GUIObject   ' object array
CALL InitObjectArray(NumberOfObjects, Objects2()) ' init array

NumberOfObjects = NumberOfFiles + 2               ' set number of objects
DIM Objects(0 TO NumberOfObjects) AS GUIObject    ' object array
CALL InitObjectArray(NumberOfObjects, Objects())  ' init array
CALL SetSemiAutoGUI(TRUE)                         ' set semi-auto mode
DIM Fil$(NumberOfFiles)                           ' dim filename array
DIM Object(NumberOfFiles)                         ' dim pointer array
DIM ULR(NumberOfFiles), ULC(NumberOfFiles)        ' position array
DIM VSCREEN(9606)                                 ' partial screen
VSEG = VARSEG(VSCREEN(1))                         ' get segment
VOFS = VARPTR(VSCREEN(1))                         ' get offset
X1 = 0: Y1 = 0: X2 = XMAX: Y2 = YMAX              ' set form coordinates

'--- draw the form (border)

CALL BOXDRAWP((X1 + 1), (Y1 + 1), (X2 - 1), (Y2 - 1), BLACK, -1, 1, 1)     ' draw border
CALL BOXDRAWP((X1 + 4), (Y1 + 3), (X2 - 4), (Y2 - 3), LIGHTGRAY, -1, 2, 3) ' draw border
CALL BOXDRAWP((X1 + 5), (Y1 + 4), (X2 - 5), (Y2 - 3), BLACK, WHITE, 1, 1)  ' draw border

'--- establish control box size

TitleX1 = X1 + 5
TitleX2 = X2 - 5
ControlBox1 = SetControlBox((TitleX1 - 1), (Y1 + 3), NORMAL, Objects())
CALL SizeControlBox((TitleX1 - 1), (Y1 + 3), NORMAL, ControlBoxHeight, ControlBoxWidth)
TitleX1 = TitleX1 + ControlBoxWidth - 1

'--- add pull down menu for exit to DOS option

ULC = X1 + 5
ULR = Y1 + 3 + ControlBoxHeight - 1
State = FOCUS                                      ' set state
Justify = LEFTJUSTIFY                              ' set justification
Caption$ = "Exit to DOS"                           ' set caption
HotColor = LIGHTRED                                ' set hot key color
Fore = BLACK: Back = WHITE                         ' set Color
CALL SizeTextBox(ULC, ULR, -1, -1, State, Justify, Caption$, HotColor, Fore, Back, TxtHeight, TxtWidth)
LRC = ULC + TxtWidth + 10                         ' set frame lower right column
LRR = ULR + TxtHeight + 10                        ' set frame lower right row
FrameBox1 = SetFrameBox(ULC, ULR, LRC, LRR, "", Back, Back, Objects2())
ULC = ULC + 1: LRC = LRC - 1: ULR = ULR + 5        ' reset frame box coordinates
TextBox1 = SetTextBox(ULC, ULR, LRC, -1, State, Justify, Caption$, HotColor, Fore, Back, Objects2())


CALL SizeFileIcon(ULC, ULR, State, "FILENAME.EXT", Fore, Back, OHeight, OWidth)                  ' get icon size
OHeight = OHeight + 2                           ' space between lines
OWidth = OWidth + 10                            ' space between lines
FrameBoxHeight = MaxIndex * OHeight + 30        ' set frame box height
FrameBoxWidth = (NumberOfColumns * OWidth) + 10 ' set frame box width
ULC = (XMAX - FrameBoxWidth) \ 2                ' upper left column
ULR = ControlBoxHeight + (YMAX - FrameBoxHeight - ControlBoxHeight) \ 2' upper left row
LRC = ULC + FrameBoxWidth - 1                   ' lower right column
LRR = ULR + FrameBoxHeight - 1                  ' lower right row
FrameColor = WHITE                              ' set frame color
Back = BLUE                                     ' set color inside frame
Fore = WHITE                                    ' set text color
FrameBox2 = SetFrameBox(ULC, ULR, LRC, LRR, "DIRECTORY", FrameColor, Back, Objects())
ULC = ULC + 10: ULR = ULR + 20                  ' indent inside box
TopRow = ULR                                    ' save top row
Pointer = 1                                     ' initialize pointer
IF DIRFIRST%(FSpec$, FAttr) = 0 THEN            ' if we found a file...
   DO                                           ' loop
      IF (Pointer > 1) THEN
        IF (((Pointer - 1) MOD IndexLimit) = 0) THEN
           ULR = TopRow                         ' start at the top row
           ULC = ULC + OWidth                   ' next column
        END IF
      END IF

      Fil$(Pointer) = DIRFILENAME$              ' get file name
      ULR(Pointer) = ULR                        ' save upper left row
      ULC(Pointer) = ULC                        ' save upper left column
      ULR = ULR + OHeight                       ' next row
      Pointer = Pointer + 1                     ' increment pointer
      IF (Pointer > NumberOfFiles) THEN EXIT DO ' all done
   LOOP UNTIL DIRNEXT                           ' til no more matches
END IF

'--- Add objects to the object array

FOR I = 1 TO NumberOfFiles
   Extention$ = RIGHT$(Fil$(I), 4)
   IF (Extention$ = ".EXE") OR (Extention$ = ".COM") OR (Extention$ = ".BAT") THEN
      Object(I) = SetExeIcon(ULC(I), ULR(I), NORMAL, Fil$(I), Fore, Back, Objects())
   ELSE
      Object(I) = SetFileIcon(ULC(I), ULR(I), NORMAL, Fil$(I), Fore, Back, Objects())
   END IF
NEXT

CALL UpdateGUI(Objects())

'--- display title on the form (screen)

CALL DrawTextBox(TitleX1, (Y1 + 3), TitleX2, (Y1 + 3 + ControlBoxHeight - 1), NORMAL, CENTERJUSTIFY, "GUI FILE/EXE ICON EXAMPLE", UNUSED, WHITE, LIGHTBLUE)

'--- draw border around text

CALL BOXDRAWP(TitleX1, (Y1 + 4), TitleX2, (Y1 + 3 + ControlBoxHeight - 2), BLACK, -1, 1, 1)

' Now let's highlight the first filename (object) in the list box:

CALL AnimateObject(Object(1), FOCUS, Objects())

CALL MMCHECK(MOUSE)
CALL MMCURSORON

DO WHILE TRUE
   CALL CheckGUIEvent(ObjectHandle, CMD1, CMD2, Objects())

   IF (ObjectHandle = ControlBox1) AND (CMD1 = MOUSEBUTTONCLICK) THEN

        CALL AnimateObject(ControlBox1, DEPRESSED, Objects())

        CALL MMCURSOROFF
        X1 = Objects2(FrameBox1).ULC
        Y1 = Objects2(FrameBox1).ULR
        X2 = Objects2(FrameBox1).LRC
        Y2 = Objects2(FrameBox1).LRR
        CALL EDGETSCREENP(VSEG, VOFS, X1, Y1, X2, Y2)   ' save screen

        CALL UpdateGUI(Objects2())
        CALL BOXDRAWP(X1 + 1, Y1 + 1, X2 - 1, Y2 - 1, BLACK, -1, 1, 1)' draw frame and frame box

        CALL FlushGUIEvent

        CALL MMCURSORON

        DO WHILE TRUE
           CALL CheckGUIEvent(ObjectHandle, CMD1, CMD2, Objects2())

           IF (ObjectHandle = TextBox1) AND (CMD1 = MOUSEBUTTONCLICK) THEN
                END
           ELSEIF (ObjectHandle = -1) AND ((CMD1 = ASC("x")) OR (CMD1 = ASC("X"))) THEN
                END
           ELSEIF (ObjectHandle = 0) THEN
                EXIT DO
           END IF
        LOOP

        CALL MMCURSOROFF
        CALL EDPUTSCREENP(VSEG, VOFS, X1, Y1, X2, Y2)
        CALL AnimateObject(ControlBox1, NORMAL, Objects())
        CALL AnimateObject(Object(1), FOCUS, Objects())
        CALL MMCURSORON
   END IF
LOOP

END

RecordModes:
  ' We come her to test modes
  UseableModes$ = " "
  FOR ModeNumber = 1 TO 13
      IF mode(ModeNumber) THEN
         UseableModes$ = UseableModes$ + STR$(ModeNumber)
      END IF
  NEXT ModeNumber
  UseableModes$ = UseableModes$ + " "
RETURN

Handler:
  ' We branch here when adapter test fails and change test result.
  mode(ModeNumber) = False
  RESUME NEXT

SetScrErr:
  ' If we can not set a screen mode revert to TEXT
  SCREEN 0
  WIDTH 80, 25
  RESUME NEXT



SUB AnimateObject (ObjectHandle%, State%, ObjectArray() AS GUIObject)
'----------------------------- AnimateObject --------------------------
'
' AnimateObject animate an object based on the ObjectHandle and State
'               variables passed.
'
' Calling Convention:
'    AnimateObject(ObjectHandle%, State%, ObjectArray() AS GUIObject)
'
'  ObjectHandle%   The Handle or Element Number of the Object in the
'                  Object array.
'  State%          New State of the Object.
'  ObjectArray()   Array of GUIObject TYPEs.
'
' returned modified       :
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     IF (ObjectHandle% < 1) OR (ObjectHandle% > ObjectArray(0).CurrentUse) THEN     ' check for invalid ObjectHandle%
        EXIT SUB
     END IF

     OH = ObjectHandle%                                          ' for brevity

     IF (ObjectArray(OH).CurrentUse = 0) THEN EXIT SUB           ' if the object is not currently used
     IF (ObjectArray(OH).Visible = FALSE) THEN EXIT SUB          ' if the object is invisible, exit

     IF (ObjectArray(OH).Enabled = TRUE) THEN
        IF (State% = ObjectArray(OH).State) THEN EXIT SUB       ' test if the state is the same
     END IF

     IF (ObjectArray(OH).TextStrLen > 0) THEN                    ' get text ready
        TempText$ = LEFT$(ObjectArray(OH).Text, ObjectArray(OH).TextStrLen)
     ELSE
        TempText$ = ""
     END IF

     SELECT CASE ObjectArray(OH).ObjectType
     CASE TextBoxType
        CALL DrawTextBox(ObjectArray(OH).ULC, ObjectArray(OH).ULR, ObjectArray(OH).LRC, ObjectArray(OH).LRR, State%, ObjectArray(OH).TextJustify, TempText$, ObjectArray(OH).HotKeyColor, ObjectArray(OH).Fore, ObjectArray(OH).Back)

     CASE FileIconType
        CALL DrawFileIcon(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%, TempText$, ObjectArray(OH).Fore, ObjectArray(OH).Back)

     CASE ExeIconType
        CALL DrawExeIcon(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%, TempText$, ObjectArray(OH).Fore, ObjectArray(OH).Back)

     CASE DirIconType
        CALL DrawDirIcon(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%, ObjectArray(OH).DirectoryType, TempText$, ObjectArray(OH).Fore, ObjectArray(OH).Back)

     CASE ComboBoxType
        CALL DrawComboBox(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%, ObjectArray(OH).LongestStringLen, TempText$)

     CASE RightArrowButType
        CALL DrawRightArrowBut(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%)

     CASE LeftArrowButType
        CALL DrawLeftArrowBut(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%)

     CASE DownArrowButType
        CALL DrawDownArrowBut(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%)

     CASE UpArrowButType
        CALL DrawUpArrowBut(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%)

     CASE HardDriveType
        CALL DrawHardDrive(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%, TempText$)

     CASE FloppyDriveType
        CALL DrawFloppyDrive(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%, TempText$)

     CASE MinMaxButType
        CALL DrawMinMaxBut(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%)

     CASE MaxButType
        CALL DrawMaxBut(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%)

     CASE MinButType
        CALL DrawMinBut(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%)

     CASE ControlBoxType
        CALL DrawControlBox(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%)

     CASE FrameBoxType
        CALL DrawFrameBox(ObjectArray(OH).ULC, ObjectArray(OH).ULR, ObjectArray(OH).LRC, ObjectArray(OH).LRR, TempText$, ObjectArray(OH).Fore, ObjectArray(OH).Back)
        ObjectArray(OH).State = NORMAL                           ' default state
        EXIT SUB

     CASE OptionButType
        CALL DrawOptionBut(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%, TempText$, ObjectArray(OH).Fore, ObjectArray(OH).Back)

     CASE CheckBoxType
        CALL DrawCheckBox(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%, TempText$, ObjectArray(OH).Fore, ObjectArray(OH).Back)

     CASE CommandButType
        CALL DrawCommandBut(ObjectArray(OH).ULC, ObjectArray(OH).ULR, State%, TempText$)

     CASE ELSE
        EXIT SUB

     END SELECT

     ObjectArray(OH).Enabled = TRUE                              ' indicate that the object has been enabled (drawn)

     ObjectArray(OH).State = State%                              ' update object state

END SUB

SUB AnimateScrollBarObject (ObjectHandle%, State%, BarPosition%, ObjectArray() AS GUIObject)
'---------------------- AnimateScrollBarObject ------------------------
'
' AnimateScrollBarObject will animate a Scroll Bar Object (either
'                        Vertical or Horizontal Scroll Bar) based on
'                        the ObjectHandle and State variables passed.
'
' Calling Convention:
'    AnimateScrollBarObject(ObjectHandle%, State%, BarPosition%,
'                           ObjectArray() AS GUIObject)
'
'  ObjectHandle%   The Handle or Element Number of the Object in the
'                  Object array.
'  State%          New State of the Object.
'  BarPosition%    The bar position for the Scroll Bar.
'  ObjectArray()   Array of GUIObject TYPEs.
'
' returned modified       :
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     CONST ScrollBarBoxSize = 21

     IF (ObjectHandle% < 1) OR (ObjectHandle% > ObjectArray(0).CurrentUse) THEN     ' check for invalid ObjectHandle%
        EXIT SUB
     END IF

     OH = ObjectHandle%                                          ' for brevity

     IF (ObjectArray(OH).CurrentUse = 0) THEN EXIT SUB           ' if the object is not currently used
     IF (ObjectArray(OH).Visible = FALSE) THEN EXIT SUB          ' if the object is invisible

     TempState = State
     IF (ObjectArray(OH).Enabled = TRUE) THEN
        IF (TempState = ObjectArray(OH).State) THEN              ' test if the state is the same
           TempState = -1                                        ' update the scroll bar only

           IF (BarPosition = ObjectArray(OH).BarPosition) THEN   ' no update neccessary
              EXIT SUB
           END IF
        END IF
     END IF

     SELECT CASE ObjectArray(OH).ObjectType
     CASE VerticalScrollBarType
        CALL DrawVerticalScrollBar(ObjectArray(OH).ULC, ObjectArray(OH).ULR, ObjectArray(OH).LRC, ObjectArray(OH).LRR, TempState, BarPosition%)

     CASE HorizontalScrollBarType
        CALL DrawHorizontalScrollBar(ObjectArray(OH).ULC, ObjectArray(OH).ULR, ObjectArray(OH).LRC, ObjectArray(OH).LRR, TempState, BarPosition%)

     CASE ELSE
        EXIT SUB

     END SELECT

     ObjectArray(OH).BarPosition = BarPosition                   ' update the scroll bar object bar position
     ObjectArray(OH).Enabled = TRUE                              ' indicate that the object has been enabled (drawn)
     IF (TempState >= 0) THEN
        ObjectArray(OH).State = TempState                        ' update the scroll bar object state
     END IF

END SUB

SUB CheckGUIEvent (ObjectHandle%, CMD1%, CMD2%, ObjectArray() AS GUIObject)
'----------------------- CheckGUIEvent --------------------------------
'
' CheckGUIEvent is similar to CheckKey() in that CheckGUIEvent() waits
'               for a keypress or a mouse click.  If a keypress, it
'               returns CMD1 and CMD2 just like CheckKey() and
'               ObjectHandle will be ZERO.  If a mouse
'               click, it checks the ObjectArray to see if the mouse is
'               located over a valid Object.  If there is a valid
'               Object, CheckGUIEvent() will return the Object Handle
'               in ObjectHandle and MOUSEBUTTONCLICK in CMD1.  If the
'               mouse event is a mouse button being depressed,
'               depending on the SemiAutoGUIFlag
'               CheckGUIEvent() will return the Object Handle in
'               ObjectHandle and MOUSEBUTTONDEPRESSED in CMD1 or
'               CheckGUIEvent() will automatically do everything.  If
'               the Object was a Scroll Bar, and the mouse depress is
'               an update to the Scroll Bar Position, that Scroll Bar
'               Position is returned in CMD2.
'               In all other cases, ObjectHandle, CMD1 and CMD2 will be
'               returned as ZERO.
'
' Calling Convention:
'    CheckGUIEvent(ObjectHandle%, CMD1%, CMD2%, ObjectArray())
'
' ObjectArray()   Array of GUIObject TYPEs.
'
' ObjectHandle Return value: as follows:
'                 ObjectHandle > 0   The array element number for that
'                                    object in the object array (Objects())
'                                    on which a GUI event took place.
'                 ObjectHandle = 0   A GUI event took place on an invalid
'                                    object.
'                 ObjectHandle = -1  A keyboard press took place.
' CMD1%        Return value.  See above.
' CMD2%        Return value.  See above.
'
' returned modified       : ObjectHandle%, CMD1%, CMD2%
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


      STATIC MouseButtonStillDepressed

      DO WHILE TRUE                                              ' wait for event
        CALL MMBUTTON(LeftDepressButton, RightDepressButton)
        IF LeftDepressButton THEN
           EXIT DO
        END IF

        CALL MMClick(LeftClickButton, RightClickButton)
        IF LeftClickButton THEN
           EXIT DO
        END IF

        CALL KEYPRESS(KeyboardHit)
        IF KeyboardHit THEN
           EXIT DO
        END IF

        MouseButtonStillDepressed = FALSE
     LOOP

     IF LeftDepressButton THEN
        CALL MMGETLOC(MouseX, MouseY)

        ObjectCount = ObjectArray(0).CurrentUse
        FOR I = 1 TO ObjectCount
           IF (ObjectArray(I).CurrentUse <> 0) AND (ObjectArray(I).Enabled = TRUE) AND (ObjectArray(I).ObjectType <> FrameBoxType) THEN
              IF (MouseX >= ObjectArray(I).ULC) AND (MouseX <= ObjectArray(I).LRC) AND (MouseY >= ObjectArray(I).ULR) AND (MouseY <= ObjectArray(I).LRR) THEN
                TempObjectType = ObjectArray(I).ObjectType
                IF (TempObjectType >= 1) AND (TempObjectType <= MaxObjectType) THEN

                TempState = ObjectArray(I).State

                SELECT CASE TempObjectType
                CASE ComboBoxType                                ' currently not available
                   ObjectHandle% = 0
                   CMD1 = 0
                   CMD2 = 0
                   MouseButtonStillDepressed = FALSE
                   EXIT SUB

                CASE VerticalScrollBarType
                   IF MouseButtonStillDepressed THEN EXIT FOR
                   IF ((ObjectArray(I).LRR - MouseY) <= ScrollBarBoxSize) THEN
                        ObjectHandle% = I
                        CMD1 = MOUSEBUTTONDEPRESSED
                        CMD2 = BOTTOMDEPRESSED
                        EXIT SUB
                   ELSEIF ((MouseY - ObjectArray(I).ULR) <= ScrollBarBoxSize) THEN
                        ObjectHandle% = I
                        CMD1 = MOUSEBUTTONDEPRESSED
                        CMD2 = TOPDEPRESSED
                        EXIT SUB
                   ELSE
                        InteriorScrollBarHeight& = (ObjectArray(I).LRR - ObjectArray(I).ULR - 2 * ScrollBarBoxSize - 1)
                        MouseY& = MouseY
                        Y1& = ObjectArray(I).ULR
                        TempBarPosition& = (1000& * (MouseY& - Y1& - ScrollBarBoxSize)) \ InteriorScrollBarHeight&

                        ObjectHandle% = I
                        CMD1 = MOUSEBUTTONDEPRESSED
                        CMD2 = TempBarPosition&
                        EXIT SUB
                   END IF

                CASE HorizontalScrollBarType
                   IF MouseButtonStillDepressed THEN EXIT FOR
                   IF ((ObjectArray(I).LRC - MouseX) <= ScrollBarBoxSize) THEN
                        ObjectHandle% = I
                        CMD1 = MOUSEBUTTONDEPRESSED
                        CMD2 = RIGHTDEPRESSED
                        EXIT SUB
                   ELSEIF ((MouseX - ObjectArray(I).ULC) <= ScrollBarBoxSize) THEN
                        ObjectHandle% = I
                        CMD1 = MOUSEBUTTONDEPRESSED
                        CMD2 = LEFTDEPRESSED
                        EXIT SUB
                   ELSE
                        InteriorScrollBarWidth& = (ObjectArray(I).LRC - ObjectArray(I).ULC - 2 * ScrollBarBoxSize - 1)
                        MouseX& = MouseX
                        X1& = ObjectArray(I).ULC
                        TempBarPosition& = (1000& * (MouseX& - X1& - ScrollBarBoxSize)) \ InteriorScrollBarWidth&

                        ObjectHandle% = I
                        CMD1 = MOUSEBUTTONDEPRESSED
                        CMD2 = TempBarPosition&
                        EXIT SUB
                   END IF

                CASE TextBoxType, DirIconType, ExeIconType, FileIconType
                   IF SemiAutoGUIFlag THEN
                        FOR J = 1 TO ObjectCount                 ' find current FOCUS and reset
                           IF ((ObjectArray(J).State AND FOCUS) <> 0) AND (J <> I) THEN
                                CALL AnimateObject(J, (ObjectArray(J).State AND (NOT (FOCUS + DEPRESSED))), ObjectArray())
                           END IF
                        NEXT J

                        TempState = TempState OR (FOCUS + DEPRESSED)
                        CALL AnimateObject(I, TempState, ObjectArray())

                        DO WHILE TRUE
                           CALL MMBUTTON(LeftButton, RightButton)
                           IF LeftButton THEN
                                CALL MMGETLOC(MouseX, MouseY)
                                IF (MouseX < ObjectArray(I).ULC) OR (MouseX > ObjectArray(I).LRC) OR (MouseY < ObjectArray(I).ULR) OR (MouseY > ObjectArray(I).LRR) THEN
                                   TempState = TempState AND (NOT DEPRESSED)
                                   CALL AnimateObject(I, TempState, ObjectArray())
                                   MouseButtonStillDepressed = TRUE

                                   ObjectHandle% = 0             ' mouse depressed on INVALID object
                                   CMD1 = MOUSEBUTTONDEPRESSED
                                   CMD2 = 0
                                   EXIT SUB
                                END IF
                           ELSE
                                CALL MMGETLOC(MouseX, MouseY)
                                IF (MouseX < ObjectArray(I).ULC) OR (MouseX > ObjectArray(I).LRC) OR (MouseY < ObjectArray(I).ULR) OR (MouseY > ObjectArray(I).LRR) THEN
                                   ObjectHandle% = 0             ' mouse click on INVALID object
                                   CMD1 = MOUSEBUTTONCLICK
                                   CMD2 = 0
                                   EXIT SUB
                                END IF

                                TempState = TempState AND (NOT DEPRESSED)
                                CALL AnimateObject(I, TempState, ObjectArray())

                                ObjectHandle = I
                                CMD1 = MOUSEBUTTONCLICK
                                CMD2 = 0
                                MouseButtonStillDepressed = FALSE
                                EXIT SUB
                           END IF
                        LOOP
                   ELSE
                        ObjectHandle% = I
                        CMD1 = MOUSEBUTTONDEPRESSED
                        CMD2 = 0
                        EXIT SUB
                   END IF

                CASE ELSE
                   IF MouseButtonStillDepressed THEN EXIT FOR
                   IF SemiAutoGUIFlag THEN
                        FOR J = 1 TO ObjectCount                 ' find current FOCUS and reset
                           IF ((ObjectArray(J).State AND FOCUS) <> 0) AND (J <> I) THEN
                                CALL AnimateObject(J, (ObjectArray(J).State AND (NOT (FOCUS + DEPRESSED))), ObjectArray())
                           END IF
                        NEXT J

                        TempState = TempState OR (FOCUS + DEPRESSED)
                        CALL AnimateObject(I, TempState, ObjectArray())

                        DO WHILE TRUE
                           CALL MMBUTTON(LeftButton, RightButton)
                           IF LeftButton THEN
                                CALL MMGETLOC(MouseX, MouseY)
                                IF (MouseX < ObjectArray(I).ULC) OR (MouseX > ObjectArray(I).LRC) OR (MouseY < ObjectArray(I).ULR) OR (MouseY > ObjectArray(I).LRR) THEN
                                   TempState = TempState AND (NOT DEPRESSED)
                                   CALL AnimateObject(I, TempState, ObjectArray())

                                   DO WHILE TRUE
                                        CALL MMBUTTON(LeftButton, RightButton)
                                        IF (LeftButton = 0) THEN
                                           EXIT DO
                                        END IF
                                   LOOP

                                   ObjectHandle% = 0             ' mouse depressed on INVALID object
                                   CMD1 = MOUSEBUTTONCLICK
                                   CMD2 = 0
                                   MouseButtonStillDepressed = FALSE
                                   EXIT SUB

                                END IF
                           ELSE
                                CALL MMGETLOC(MouseX, MouseY)
                                IF (MouseX < ObjectArray(I).ULC) OR (MouseX > ObjectArray(I).LRC) OR (MouseY < ObjectArray(I).ULR) OR (MouseY > ObjectArray(I).LRR) THEN
                                   ObjectHandle% = 0             ' mouse click on INVALID object
                                   CMD1 = MOUSEBUTTONCLICK
                                   CMD2 = 0
                                   MouseButtonStillDepressed = FALSE
                                   EXIT SUB
                                END IF

                                TempState = TempState AND (NOT DEPRESSED)
                                CALL AnimateObject(I, TempState, ObjectArray())

                                ObjectHandle = I
                                CMD1 = MOUSEBUTTONCLICK
                                CMD2 = 0
                                MouseButtonStillDepressed = FALSE
                                EXIT SUB
                           END IF
                        LOOP
                   ELSE
                        ObjectHandle% = I
                        CMD1 = MOUSEBUTTONDEPRESSED
                        CMD2 = 0
                        EXIT SUB
                   END IF
                END SELECT
                END IF
              END IF
           END IF
        NEXT

        ObjectHandle% = 0                                        ' mouse depressed on INVALID object
        CMD1 = MOUSEBUTTONDEPRESSED
        CMD2 = 0
        EXIT SUB

     ELSEIF LeftClickButton THEN
        MouseButtonStillDepressed = FALSE
        CALL MMCLICKLOCL(MouseX, MouseY)

        ObjectCount = ObjectArray(0).CurrentUse
        FOR I = 1 TO ObjectCount
           IF (ObjectArray(I).CurrentUse <> 0) AND (ObjectArray(I).Enabled = TRUE) AND (ObjectArray(I).ObjectType <> FrameBoxType) THEN
                IF (MouseX >= ObjectArray(I).ULC) AND (MouseX <= ObjectArray(I).LRC) AND (MouseY >= ObjectArray(I).ULR) AND (MouseY <= ObjectArray(I).LRR) THEN
                   ObjectHandle% = I
                   CMD1 = MOUSEBUTTONCLICK
                   CMD2 = 0
                   EXIT SUB
                END IF
           END IF
        NEXT

        ObjectHandle% = 0                                         ' mouse click on INVALID object
        CMD1 = 0
        CMD2 = 0
        EXIT SUB
     ELSE
        Dummy = 0
        CALL GETKEY(Dummy, ASCIICode, ScanCode, Dummy, Dummy)    ' keyboard control
        ObjectHandle = -1
        CMD1 = ASCIICode
        CMD2 = ScanCode
        EXIT SUB
     END IF

END SUB

SUB DeleteObject (ObjectHandle%, ObjectArray() AS GUIObject)
'--------------------------- DeleteObject -----------------------------
'
' DeleteObject will delete the Object with the passed Handle from the
'              GUIObject Array.
'
' Calling Convention:
'         DeleteObject(ObjectHandle%, ObjectArray() AS GUIObject)
'
'  ObjectHandle%   The Handle or Element Number of the Object in the
'                  Object array.
'  ObjectArray()   Array of GUIObject TYPEs.
'
' returned modified       :
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     '*
     '*  Check for invalid ObjectHandle%
     '*
     IF (ObjectHandle% < 1) OR (ObjectHandle% > ObjectArray(0).CurrentUse) THEN
        EXIT SUB
     END IF

     '*
     '*  Delete Object from the Array
     '*
     ObjectArray(ObjectHandle%).CurrentUse = 0
     ObjectArray(ObjectHandle%).Enabled = FALSE
     ObjectArray(ObjectHandle%).Visible = FALSE

END SUB

SUB DrawCheckBox (ULC%, ULR%, State%, Caption$, Fore%, Back%)
' -------------------------- DrawCheckBox --------------------------------
'
'  DrawCheckBox will draw a GUI Check Box at the passed
'                 coordinates with the passed State and Caption.
'
'  Calling Convention:
'  DrawCheckBox(ULC%, ULR%, State%, Caption$, Fore%, Back%)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                                   Non-Selected (Empty Check Box)
'                      State = 0    Normal State
'                      State = 1    Command Button Depressed
'                      State = 2    Command Button with Focus
'                      State = 3    Commnad Button Depressed with Focus *
'                                   Selected (X in Check Box)
'                      State = 4    Normal State
'                      State = 5    Command Button Depressed
'                      State = 6    Command Button with Focus
'                      State = 7    Commnad Button Depressed with Focus *
'  Caption$     Text for Check Box
'  Fore%        Foreground Color (0 <= Fore% <= 15)
'  Back%        Background Color (0 <= Back% <= 15)
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SELECT CASE GETCHARSIZE                                     ' set font height according to CHARSIZE
     CASE 0
        FontHeight = 8
        CheckBoxSize = 10
        CharX = 16
        CharY = 2
     CASE 1
        FontHeight = 14
        CheckBoxSize = 14
        CharX = 22
        CharY = 2
     CASE 2
        FontHeight = 16
        CheckBoxSize = 16
        CharX = 24
        CharY = 3
     CASE ELSE
        EXIT SUB
     END SELECT

     CALL MMCURSOROFF

     CaptionLen = 8 * LEN(Caption$)

     X1 = ULC                                                    ' Set the CheckBox coordinates
     Y1 = ULR
     X2 = ULC + CheckBoxSize - 1
     Y2 = ULR + CheckBoxSize - 1

     CALL BOXDRAWP((X1 + 1), (Y1 + 1), (X2 - 1), (Y2 - 1), Fore%, Back%, 1, 1)' start drawing the CheckBox

     IF (State AND SELECTED) THEN
        CALL ELINE(X1, Y1, X2, Y2, Fore%)
        CALL ELINE(X1, Y2, X2, Y1, Fore%)
     END IF

     IF (State AND DEPRESSED) THEN
        CALL BOXDRAWP((X1 + 2), (Y1 + 2), (X2 - 2), (Y2 - 2), Fore%, -1, 1, 1)
     END IF

     CharX = X1 + CharX
     CharY = Y1 + CharY
     CALL EXQPRINTP(Caption$, CharX, CharY, Fore%, Back%)

     IF (State AND FOCUS) THEN
        CALL FocusBox((CharX - 3), (CharY - 2), (CharX + CaptionLen + 3), (CharY + FontHeight), Fore%)
     ELSE
        CALL FocusBox((CharX - 3), (CharY - 2), (CharX + CaptionLen + 3), (CharY + FontHeight), Back%)
     END IF

     CALL MMCURSORON

END SUB

SUB DrawComboBox (ULC%, ULR%, State%, LongestStringLen%, Caption$)
'------------------------- DrawComboBox ----------------------------------
'
'  DrawComboBox will draw a GUI Combo Box at the passed
'               coordinates with the passed State and Caption.
'
'  Calling Convention:
'  DrawComboBox(ULC%, ULR%, State%, LongestStringLen%, Caption$)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                       State = 0    Normal Combo Box
'                       State = 1    Depressed Combo Box
'                       State = 2    Combo Box with Focus
'                       State = 3    Depressed Combo Box with Focus
'                                    Update Combo Box Button Only
'                       State = 8    Normal Combo Box
'                       State = 9    Depressed Combo Box
'                       State = 10   Combo Box with Focus
'                       State = 11   Depressed Combo Box with Focus
'  LongestStringLen% The character length of the longest string for the
'                    item list of Combo Box.
'  Caption$          Current Caption in the Combo Box from the item list *
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     DIM LineMask(1 TO 12) AS INTEGER

     SELECT CASE GETCHARSIZE                                     ' set ComboBox button size according to the current CHARSIZE
     CASE 0
        FontHeight = 8
        BoxHeight = 16
        YIndent = 4

        LineMask(1) = &H3800
        LineMask(2) = &H3800
        LineMask(3) = &HFE00
        LineMask(4) = &H7C00
        LineMask(5) = &H3800
        LineMask(6) = &H1000
        LineMask(7) = &H0
        LineMask(8) = &HFE00
        LineMask(9) = &H0
        LineMask(10) = &H0
        LineMask(11) = &H0
        LineMask(12) = &H0

        ComboButtonWidth = 7

     CASE 1
        FontHeight = 14
        BoxHeight = 20
        YIndent = 5

        LineMask(1) = &H1C00
        LineMask(2) = &H1C00
        LineMask(3) = &H1C00
        LineMask(4) = &HFF80
        LineMask(5) = &H7F00
        LineMask(6) = &H3E00
        LineMask(7) = &H1C00
        LineMask(8) = &H800
        LineMask(9) = &H0
        LineMask(10) = &HFF80
        LineMask(11) = &H0
        LineMask(12) = &H0

        ComboButtonWidth = 9

     CASE 2
        FontHeight = 16
        BoxHeight = 22
        YIndent = 5

        LineMask(1) = &H1F00
        LineMask(2) = &H1F00
        LineMask(3) = &H1F00
        LineMask(4) = &H1F00
        LineMask(5) = &HFFE0
        LineMask(6) = &H7FC0
        LineMask(7) = &H3F80
        LineMask(8) = &H1F00
        LineMask(9) = &HE00
        LineMask(10) = &H400
        LineMask(11) = &H0
        LineMask(12) = &HFFE0

        ComboButtonWidth = 11

     CASE ELSE
        EXIT SUB
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        BoxWidth = BoxHeight + 3
     CASE 18
        BoxWidth = BoxHeight + 1
     CASE 91
        BoxWidth = BoxHeight + 1
     CASE ELSE
        EXIT SUB
     END SELECT
     XIndent = (BoxWidth - ComboButtonWidth) \ 2

     CALL MMCURSOROFF

     TextBoxLen = (LongestStringLen% + 2) * 8                    ' start drawing the ComboBox
     TempX1 = ULC + 1
     TempY1 = ULR + 1
     TempX2 = ULC + TextBoxLen - 1 - 1
     TempY2 = ULR + BoxHeight - 1 - 1

     IF ((State AND UPDATEONLY) = 0) THEN
        CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, WHITE, 1, 1)
        CharX = ULC + 8
        CharY = ULR + ((BoxHeight - FontHeight) \ 2)
        IF (State AND FOCUS) THEN
             CALL EXQPRINTP(Caption$, CharX, CharY, WHITE, BLACK)
        ELSE
             CALL EXQPRINTP(Caption$, CharX, CharY, BLACK, WHITE)
        END IF
     END IF

     TempX1 = TempX2 + (BoxWidth \ 2)
     IF (State AND DEPRESSED) THEN
        CALL DrawGenericButton(TempX1, ULR, (TempX1 + BoxWidth - 1), (ULR + BoxHeight - 1), DEPRESSED)

        TempX1 = TempX1 + XIndent + 1
        TempY1 = ULR + YIndent + 1
        TempX2 = TempX1 + ComboButtonWidth
     ELSE
        CALL DrawGenericButton(TempX1, ULR, (TempX1 + BoxWidth - 1), (ULR + BoxHeight - 1), NORMAL)

        TempX1 = TempX1 + XIndent
        TempY1 = ULR + YIndent
        TempX2 = TempX1 + ComboButtonWidth
     END IF

     FOR I = 1 TO 12                                             ' bit mask for ComboBox down arrow
        IF (LineMask(I)) THEN
             CALL SETLINESTYLE(LineMask(I))
             CALL ELINE(TempX1, TempY1, TempX2, TempY1, BLACK)
        END IF
        TempY1 = TempY1 + 1
     NEXT
     CALL SETLINESTYLE(&HFFFF)

     CALL MMCURSORON

END SUB

SUB DrawCommandBut (ULC%, ULR%, State%, Caption$)
' ------------------------ DrawCommandBut --------------------------------
'
'  DrawCommandBut will draw a GUI Command Button at the passed
'                 coordinates with the passed State and Caption.
'
'  Calling Convention:
'  DrawCommandBut(ULC%, ULR%, State%, Caption%)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                      State = 0    Normal State
'                      State = 1    Command Button Depressed
'                      State = 2    Command Button with Focus
'                      State = 3    Commnad Button Depressed with Focus *
'  Caption$     Text for Command Button
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SELECT CASE GETCHARSIZE                                     ' set font height according to CHARSIZE
     CASE 0
        FontHeight = 8
     CASE 1
        FontHeight = 14
     CASE 2
        FontHeight = 16
     CASE ELSE
        EXIT SUB
     END SELECT

     CALL MMCURSOROFF

     CaptionLen = 8 * LEN(Caption$)                              ' set the CommandBut width and height
     CommandButtonHeight = 18 + FontHeight
     CommandButtonWidth = CaptionLen + 40

     X1 = ULC
     Y1 = ULR
     X2 = X1 + CommandButtonWidth
     Y2 = Y1 + CommandButtonHeight

     CALL ELINE((X1 + 1), Y1, (X2 - 1), Y1, BLACK)               ' start drawing the CommandBut
     CALL ELINE((X1 + 1), Y2, (X2 - 1), Y2, BLACK)
     CALL ELINE(X1, (Y1 + 1), X1, (Y2 - 1), BLACK)
     CALL ELINE(X2, (Y1 + 1), X2, (Y2 - 1), BLACK)

     X1 = X1 + 1:  Y1 = Y1 + 1:  X2 = X2 - 1:  Y2 = Y2 - 1

     IF (State AND FOCUS) THEN               ' Focus State
        X1 = X1 + 1:  Y1 = Y1 + 1:  X2 = X2 - 1:  Y2 = Y2 - 1
        CALL BOXDRAWP(X1, Y1, X2, Y2, BLACK, -1, 1, 1)
     END IF

     CALL BOXDRAWP(X1, Y1, X2, Y2, LIGHTGRAY, LIGHTGRAY, 0, 0)

     IF (State AND DEPRESSED) THEN
        CALL ELINE(X1, Y1, X2, Y1, DARKGRAY)
        CALL ELINE(X1, Y1, X1, Y2, DARKGRAY)

        X1 = X1 + 1:  Y1 = Y1 + 1
     ELSE
        CALL ELINE(X1, Y1, (X2 - 1), Y1, WHITE)
        CALL ELINE(X1, Y1, X1, (Y2 - 1), WHITE)

        CALL ELINE((X1 + 1), Y2, X2, Y2, DARKGRAY)
        CALL ELINE(X2, (Y1 + 1), X2, Y2, DARKGRAY)

        X1 = X1 + 1:  Y1 = Y1 + 1:  X2 = X2 - 1:  Y2 = Y2 - 1

        CALL ELINE(X1, Y1, (X2 - 1), Y1, WHITE)
        CALL ELINE(X1, Y1, X1, (Y2 - 1), WHITE)

        CALL ELINE((X1 + 1), Y2, X2, Y2, DARKGRAY)
        CALL ELINE(X2, (Y1 + 1), X2, Y2, DARKGRAY)

        X1 = X1 + 1:  Y1 = Y1 + 1:  X2 = X2 - 1:  Y2 = Y2 - 1
     END IF

     CharX = X1 + (X2 - X1 - CaptionLen + 1) \ 2
     CharY = Y1 + 1 + (Y2 - Y1 - FontHeight + 1) \ 2

     CALL EXQPRINTP(Caption$, CharX, CharY, BLACK, LIGHTGRAY)

     IF (State AND FOCUS) THEN
        CALL FocusBox((CharX - 3), (CharY - 2), (CharX + CaptionLen + 3), (CharY + FontHeight), DARKGRAY)
     END IF

     CALL MMCURSORON

END SUB

SUB DrawControlBox (ULC%, ULR%, State%)
' ------------------------ DrawControlBox --------------------------------
'
'  DrawControlBox will draw a GUI Control Box at the passed
'                      coordinates with the passed State.
'
'  Calling Convention: DrawControlBox(ULC%, ULR%, State%)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                       State = 0    Normal Control Box
'                       State = 1    Depressed Control Box
'
' returned modified       :
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SELECT CASE GETCHARSIZE                                     ' set ControlBox size according to the current CHARSIZE
     CASE 0
        ControlBoxHeight = 16
        YIndent = 6
     CASE 1
        ControlBoxHeight = 20
        YIndent = 8
     CASE 2
        ControlBoxHeight = 22
        YIndent = 9
     CASE ELSE
        EXIT SUB
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        ControlBoxWidth = ControlBoxHeight + 3
     CASE 18
        ControlBoxWidth = ControlBoxHeight + 1
     CASE 91
        ControlBoxWidth = ControlBoxHeight + 1
     CASE ELSE
        EXIT SUB
     END SELECT
     BarLen = ControlBoxWidth - 8

     CALL MMCURSOROFF

     IF (State AND DEPRESSED) THEN                               ' start drawing the ControlBox
        CALL BOXDRAWP((ULC + 1), (ULR + 1), (ULC + ControlBoxWidth - 2), (ULR + ControlBoxHeight - 2), BLACK, DARKGRAY, 1, 1)

        X1 = ULC + 4 + 1
        Y1 = ULR + YIndent + 1
        CALL BOXDRAWP(X1, Y1, (X1 + BarLen - 3), Y1, WHITE, BLACK, 1, 1)
     ELSE
        CALL BOXDRAWP((ULC + 1), (ULR + 1), (ULC + ControlBoxWidth - 2), (ULR + ControlBoxHeight - 2), BLACK, LIGHTGRAY, 1, 1)

        X1 = ULC + 4 + 1
        Y1 = ULR + YIndent + 1
        CALL BOXDRAWP(X1, Y1, (X1 + BarLen - 3), Y1, BLACK, WHITE, 1, 1)
        CALL ELINE(X1, (Y1 + 2), (X1 + BarLen - 1), (Y1 + 2), DARKGRAY)
        CALL ELINE((X1 + BarLen - 1), (Y1 - 1), (X1 + BarLen - 1), (Y1 + 2), DARKGRAY)
     END IF

     CALL MMCURSORON

END SUB

SUB DrawDirIcon (ULC%, ULR%, State%, DirType%, DirName$, Fore%, Back%)
'------------------------- DrawDirIcon ----------------------------------
'
'  DrawDirIcon will draw a GUI File Manager Dir Icon at the
'              passed coordinates with the passed State, Dir Type and
'              Dir Name.
'
'  Calling Convention:
'  DrawDirIcon(ULC%, ULR%, State%, DirType%, DirName$, Fore%, Back%)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                       State = 0    Normal Directory Icon
'                       State = 2    Directory Icon with Focus
'  DirType%     As follows:
'                       DirType = 0  Normal Directory Icon
'                       DirType = 1  Directory Icon with "-".  Used to
'                                    denote Parent Directory.
'                       DirType = 2  Directory Icon with "+".  Used to
'                                    denote current directory has
'                                    sub-directories.
' DirName$      Text Directory Name associated with the Icon
' Fore%         Foreground Color (0 <= Fore% <= 15)
' Back%         Background Color (0 <= Back% <= 15)
'
' returned modified       :
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     TempLen = LEN(DirName$)                                     ' check for invalid DirName length
     IF (TempLen > 13) THEN
        EXIT SUB
     END IF

     IF (DirType > 2) OR (DirType < 0) THEN                      ' check for invalid DirType
        EXIT SUB
     END IF

     SELECT CASE GETCHARSIZE                                      ' set font height
     CASE 0
        FontHeight = 8
        CharY = 7
     CASE 1
        FontHeight = 14
        CharY = 3
     CASE 2
        FontHeight = 16
        CharY = 2
     CASE ELSE
        EXIT SUB
     END SELECT

     CALL MMCURSOROFF

     IF (State AND FOCUS) THEN                                   ' draw BLACK background if FOCUS
        CALL BOXDRAWP(ULC, ULR, (ULC + 146), (ULR + 17), Fore%, Fore%, 0, 0)
     ELSE
        CALL BOXDRAWP(ULC, ULR, (ULC + 146), (ULR + 17), Back%, Back%, 0, 0)
     END IF

     X1 = ULC + 4                                                ' start drawing the DirIcon
     Y1 = ULR + 2
     TempX1 = X1 + 1 + 1
     TempY1 = Y1 + 3 + 1
     TempX2 = X1 + 25
     TempY2 = Y1 + 12
     IF (State AND FOCUS) THEN
        CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, DARKGRAY, 1, 1)
     ELSE
        CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, DARKGRAY, 0, 0)
     END IF

     TempX1 = X1 + 1
     TempY1 = Y1 + 3 + 1
     TempX2 = X1 + 23
     TempY2 = Y1 + 10
     CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, LIGHTGRAY, 1, 1)

     TempY1 = Y1                                                 ' draw irregularities by drawing line segments
     CALL ELINE((X1 + 1), TempY1, (X1 + 8), TempY1, BLACK)

     TempY1 = TempY1 + 1                                         ' draw irregularities by drawing line segments
     CALL ELINE((X1 + 0), TempY1, (X1 + 9), TempY1, BLACK)
     CALL ELINE((X1 + 2), TempY1, (X1 + 7), TempY1, LIGHTGRAY)

     TempY1 = TempY1 + 1                                         ' draw irregularities by drawing line segments
     CALL ELINE((X1 + 0), TempY1, (X1 + 23), TempY1, BLACK)
     CALL ELINE((X1 + 1), TempY1, (X1 + 8), TempY1, LIGHTGRAY)

     TempY1 = TempY1 + 1                                         ' draw irregularities by drawing line segments
     CALL ELINE((X1 + 10), TempY1, (X1 + 22), TempY1, LIGHTGRAY)

     IF (DirType = 1) OR (DirType = 2) THEN                      ' draw DirType:  "-", "+", or nothing
        TempX1 = X1 + 11
        TempY1 = Y1 + 7
        TempX2 = X1 + 15
        TempY2 = Y1 + 7
        CALL ELINE(TempX1, TempY1, TempX2, TempY2, BLACK)
     END IF
     IF (DirType = 2) THEN
        TempX1 = X1 + 13
        TempY1 = Y1 + 5
        TempX2 = X1 + 13
        TempY2 = Y1 + 9
        CALL ELINE(TempX1, TempY1, TempX2, TempY2, BLACK)
     END IF

     CharX = ULC + 35
     CharY = ULR + CharY
     IF (State AND FOCUS) THEN
        CALL EXQPRINTP(DirName$, CharX, CharY, Back%, Fore%)
        CALL FocusBox(ULC, ULR, (ULC + 146), (ULR + 17), Back%)
     ELSE
        CALL EXQPRINTP(DirName$, CharX, CharY, Fore%, Back%)
     END IF

     CALL MMCURSORON

END SUB

SUB DrawDownArrowBut (ULC%, ULR%, State%)
'------------------------ DrawDownArrowBut -------------------------------
'
'  DrawDownArrowBut will draw a GUI Down Arrow Button at the
'                   passed coordinates with the passed State.
'
'  Calling Convention: DrawDownArrowBut(ULC%, ULR%, State%)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                       State = 0    Normal Down Arrow Button
'                       State = 1    Depressed Down Arrow Button
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     DIM LineMask(1 TO 5) AS INTEGER

     CALL GetVideoMode(CurrentVideoMode)                         ' set DownArrow button size according to the current video mode
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        BoxHeight = 10
        BoxWidth = 13
        XIndent = 3
        YIndent = 3

        LineMask(1) = &HFE00
        LineMask(2) = &H7C00
        LineMask(3) = &H3800
        LineMask(4) = &H1000
        LineMask(5) = &H0

        ArrowWidth = 7

     CASE 18
        BoxHeight = 10
        BoxWidth = 11
        XIndent = 2
        YIndent = 3

        LineMask(1) = &HFE00
        LineMask(2) = &H7C00
        LineMask(3) = &H3800
        LineMask(4) = &H1000
        LineMask(5) = &H0

        ArrowWidth = 7

     CASE 91
        BoxHeight = 11
        BoxWidth = 15
        XIndent = 3
        YIndent = 3

        LineMask(1) = &HFF80
        LineMask(2) = &H7F00
        LineMask(3) = &H3E00
        LineMask(4) = &H1C00
        LineMask(5) = &H800

        ArrowWidth = 9

     CASE ELSE
        EXIT SUB
     END SELECT

     CALL MMCURSOROFF

     CALL BOXDRAWP((ULC + 1), (ULR + 1), (ULC + BoxWidth - 2), (ULR + BoxHeight - 2), BLACK, LIGHTGRAY, 1, 1)' start drawing the DownArrow
     IF (State AND DEPRESSED) THEN
        ArrowColor = WHITE
     ELSE
        ArrowColor = BLACK
     END IF

     TempX1 = ULC + XIndent
     TempY = ULR + YIndent
     TempX2 = TempX1 + ArrowWidth - 1
     FOR I = 1 TO 5
        CALL SETLINESTYLE(LineMask(I))
        CALL ELINE(TempX1, TempY, TempX2, TempY, ArrowColor)
        TempY = TempY + 1
     NEXT
     CALL SETLINESTYLE(&HFFFF)

     CALL MMCURSORON

END SUB

SUB DrawExeIcon (ULC%, ULR%, State%, FileName$, Fore%, Back%)
'------------------------- DrawExeIcon ----------------------------------
'
'  DrawExeIcon will draw a GUI File Manager Executeable Icon
'              at the passed coordinates with the passed State and
'              File Name.
'
'  Calling Convention:
'  DrawExeIcon(ULC%, ULR%, State%, FileName$, Fore%, Back%)
'
'  ULC%          Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%          Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%        As follows:
'                       State = 0    Normal Executeable Icon
'                       State = 2    Executeable Icon with Focus
'  FileName$     Text Executeable File Name associated with the Icon
'  Fore%         Foreground Color (0 <= Fore% <= 15)
'  Back%         Background Color (0 <= Back% <= 15)
'
' returned modified       :
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     TempLen = LEN(FileName$)                                    ' check for invalid FileName length
     IF (TempLen > 13) THEN
        EXIT SUB
     END IF

     SELECT CASE GETCHARSIZE                                     ' set font height
     CASE 0
        FontHeight = 8
        CharY = 7
     CASE 1
        FontHeight = 14
        CharY = 3
     CASE 2
        FontHeight = 16
        CharY = 2
     CASE ELSE
        EXIT SUB
     END SELECT

     CALL MMCURSOROFF

     IF (State AND FOCUS) THEN                                   ' draw BLACK background if FOCUS
        CALL BOXDRAWP(ULC, ULR, (ULC + 146), (ULR + 17), Fore%, Fore%, 0, 0)
     ELSE
        CALL BOXDRAWP(ULC, ULR, (ULC + 146), (ULR + 17), Back%, Back%, 0, 0)
     END IF

     X1 = ULC + 5                                                ' start drawing the ExeIcon
     Y1 = ULR + 2
     TempX1 = X1 + 1 + 1
     TempY1 = Y1 + 1 + 1
     TempX2 = X1 + 23
     TempY2 = Y1 + 12
     IF (State AND FOCUS) THEN
        CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, DARKGRAY, 1, 1)
     ELSE
        CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, DARKGRAY, 0, 0)
     END IF

     TempX1 = X1 + 1
     TempY1 = Y1 + 1
     TempX2 = X1 + 21
     TempY2 = Y1 + 10
     CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, LIGHTGRAY, 1, 1)

     TempX1 = X1 + 1
     TempY1 = Y1 + 1
     TempX2 = X1 + 21
     TempY2 = TempY1 + 1
     CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, CYAN, 1, 1)

     CharX = ULC + 35
     CharY = ULR + CharY
     IF (State AND FOCUS) THEN
        CALL EXQPRINTP(FileName$, CharX, CharY, Back%, Fore%)
        CALL FocusBox(ULC, ULR, (ULC + 146), (ULR + 17), Back%)
     ELSE
        CALL EXQPRINTP(FileName$, CharX, CharY, Fore%, Back%)
     END IF

     CALL MMCURSORON

END SUB

SUB DrawFileIcon (ULC%, ULR%, State%, FileName$, Fore%, Back%)
'------------------------- DrawFileIcon ----------------------------------
'
'  DrawFileIcon will draw a GUI File Manager File Icon at the
'              passed coordinates with the passed State and File Name.
'
'  Calling Convention:
'  DrawFileIcon(ULC%, ULR%, State%, FileName$, Fore%, Back%)
'
'  ULC%          Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%          Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%        As follows:
'                       State = 0    Normal File Icon
'                       State = 2    File Icon with Focus
'  FileName$     Text File Name associated with the Icon
'  Fore%         Foreground Color (0 <= Fore% <= 15)
'  Back%         Background Color (0 <= Back% <= 15)
'
' returned modified       :
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     DIM BlackLineMask(1 TO 7) AS INTEGER
     DIM WhiteLineMask(1 TO 7) AS INTEGER
     DIM BackLineMask(1 TO 7) AS INTEGER

     TempLen = LEN(FileName$)                                    ' check for invalid FileName length
     IF (TempLen > 13) THEN
        EXIT SUB
     END IF

     BlackLineMask(1) = &H8000                                   ' set line masks
     BlackLineMask(2) = &HC000
     BlackLineMask(3) = &HE000
     BlackLineMask(4) = &HB000
     BlackLineMask(5) = &H9800
     BlackLineMask(6) = &H8C00
     BlackLineMask(7) = &HFC00

     WhiteLineMask(1) = &H0
     WhiteLineMask(2) = &H0
     WhiteLineMask(3) = &H0
     WhiteLineMask(4) = &H4000
     WhiteLineMask(5) = &H6000
     WhiteLineMask(6) = &H7000
     WhiteLineMask(7) = &H0

     BackLineMask(1) = &H7C00
     BackLineMask(2) = &H3C00
     BackLineMask(3) = &H1C00
     BackLineMask(4) = &HC00
     BackLineMask(5) = &H400
     BackLineMask(6) = &H0
     BackLineMask(7) = &H0

     SELECT CASE GETCHARSIZE                                     ' set font height
     CASE 0
        FontHeight = 8
        CharY = 7
     CASE 1
        FontHeight = 14
        CharY = 3
     CASE 2
        FontHeight = 16
        CharY = 2
     CASE ELSE
        EXIT SUB
     END SELECT

     CALL MMCURSOROFF

     TempBack = Back%                                            ' draw BLACK background if FOCUS
     IF (State AND FOCUS) THEN
        TempBack = Fore%
     END IF
     CALL BOXDRAWP(ULC, ULR, (ULC + 146), (ULR + 17), TempBack, TempBack, 0, 0)

     X1 = ULC + 9                                                ' start drawing the FileIcon
     Y1 = ULR + 2
     TempX1 = X1 + 1 + 1
     TempY1 = Y1 + 7 + 1
     TempX2 = X1 + 15
     TempY2 = Y1 + 12
     IF (State AND FOCUS) THEN
        CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, DARKGRAY, 1, 1)
     ELSE
        CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, DARKGRAY, 0, 0)
     END IF

     TempX1 = X1 + 1
     TempY1 = Y1 + 1
     TempX2 = X1 + 13
     TempY2 = Y1 + 10
     CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, LIGHTGRAY, 1, 1)

     TempY1 = Y1                                                 ' draw irregularities by drawing line segments
     TempX1 = X1 + 9
     TempX2 = X1 + 14
     FOR I = 1 TO 7
        IF (BlackLineMask(I)) THEN
           CALL SETLINESTYLE(BlackLineMask(I))
           CALL ELINE(TempX1, TempY1, TempX2, TempY1, BLACK)
        END IF
        IF (WhiteLineMask(I)) THEN
           CALL SETLINESTYLE(WhiteLineMask(I))
           CALL ELINE(TempX1, TempY1, TempX2, TempY1, WHITE)
        END IF
        IF (BackLineMask(I)) THEN
           CALL SETLINESTYLE(BackLineMask(I))
           CALL ELINE(TempX1, TempY1, TempX2, TempY1, TempBack)
        END IF
        TempY1 = TempY1 + 1
     NEXT
     CALL SETLINESTYLE(&HFFFF)

     CharX = ULC + 35
     CharY = ULR + CharY
     IF (State AND FOCUS) THEN
        CALL EXQPRINTP(FileName$, CharX, CharY, Back%, Fore%)
        CALL FocusBox(ULC, ULR, (ULC + 146), (ULR + 17), Back%)
     ELSE
        CALL EXQPRINTP(FileName$, CharX, CharY, Fore%, Back%)
     END IF

     CALL MMCURSORON

END SUB

SUB DrawFloppyDrive (ULC%, ULR%, State%, DriveLetter$)
'------------------------ DrawFloppyDrive --------------------------------
'
'  DrawFloppyDrive will draw a GUI Floppy Drive at the passed
'                  coordinates with the passed State.
'
'  Calling Convention:
'  DrawFloppyDrive(ULC%, ULR%, State%, DriveLetter$)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                       State = 0    Normal Floppy Drive
'                       State = 2    Floppy Drive with FOCUS
'  DriveLetter$ Text for the Drive Letter
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     CONST DRIVEWIDTH = 40
     CONST DRIVEHEIGHT = 17

     DIM LineMask(1 TO 6) AS INTEGER
     LineMask(1) = &HC
     LineMask(2) = &HC
     LineMask(3) = &H0
     LineMask(4) = &HFF80
     LineMask(5) = &H0
     LineMask(6) = &HFF80

     SELECT CASE GETCHARSIZE                                     ' get font size according to the current CHARSIZE
     CASE 0
        FontHeight = 8
     CASE 1
        FontHeight = 14
     CASE 2
        FontHeight = 16
     CASE ELSE
        EXIT SUB
     END SELECT

     CALL MMCURSOROFF

     TempX1 = ULC + 1                                            ' start drawing the FloppyDrive
     TempY1 = ULR + 1
     TempX2 = ULC + 69 - 1 - 1
     TempY2 = ULR + 27 - 1 - 1

     IF (State AND FOCUS) THEN
        CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, BLACK, 1, 1)
     ELSE
        CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, WHITE, 1, 1)
     END IF

     X1 = ULC + 8
     Y1 = ULR + 5
     CALL DrawGenericDrive(X1, Y1)

     TempX1 = X1 + 16
     TempY1 = Y1 + 4
     TempX2 = TempX1 + 16 - 1
     FOR I = 1 TO 6
        IF (LineMask(I)) THEN
             CALL SETLINESTYLE(LineMask(I))
             CALL ELINE(TempX1, TempY1, TempX2, TempY1, BLACK)
        END IF
        TempY1 = TempY1 + 1
     NEXT
     CALL SETLINESTYLE(&HFFFF)

     TempX1 = X1 + 4
     TempY1 = Y1 + 8
     TempX2 = X1 + DRIVEWIDTH - 6
     CALL ELINE(TempX1, TempY1, TempX2, TempY1, BLACK)

     TempX1 = ULC + 54
     TempY1 = ULR + ((28 - FontHeight) \ 2)
     TempDriveLetter$ = MID$(DriveLetter$, 1, 1)
     IF (State AND FOCUS) THEN
        CALL EXQPRINTP(TempDriveLetter$, TempX1, TempY1, LIGHTGRAY, BLACK)
     ELSE
        CALL EXQPRINTP(TempDriveLetter$, TempX1, TempY1, BLACK, WHITE)
     END IF

     CALL MMCURSORON

END SUB

SUB DrawFrameBox (ULC%, ULR%, LRC%, LRR%, FrameTitle$, FrameColor%, FillColor%)
' ------------------------ DrawFrameBox ----------------------------------
'
'  DrawFrameBox will draw a GUI Frame Box at the passed
'               coordinates with the passed colors and optional Caption. *
'
'  Calling Convention:
'  DrawFrameBox(ULC%, ULR%, LRC%, LRR%, FrameTitle$, FrameColor%, FillColor%)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  LRC%         Bottom right corner X coordinate (0 <= LRC <= XMAX)
'  LRR%         Bottom right corner Y coordinate (0 <= LRR <= YMAX)
'  FrameTitle$  Text for the Frame Title
'  FrameColor%  Color for the Frame Border
'  FillColor%   Color for the interior of the Frame Box
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SELECT CASE GETCHARSIZE                                     ' set font height according to the current video mode
     CASE 0
        FontHeight = 8
        Indent = 4
     CASE 1
        FontHeight = 14
        Indent = 6
     CASE 2
        FontHeight = 16
        Indent = 7
     CASE ELSE
        EXIT SUB
     END SELECT

     CALL MMCURSOROFF

     IF (ULC < LRC) THEN
        X1 = ULC
        X2 = LRC
     ELSE
        X1 = LRC
        X2 = ULC
     END IF

     IF (ULR < LRR) THEN
        Y1 = ULR
        Y2 = LRR
     ELSE
        Y1 = LRR
        Y2 = ULR
     END IF

     CALL BOXDRAWP(X1, Y1, X2, Y2, FillColor%, FillColor%, 0, 0)
     TempX1 = X1 + Indent
     TempY1 = Y1 + Indent
     X2 = X2 - Indent
     Y2 = Y2 - Indent
     CALL BOXDRAWP(TempX1, TempY1, X2, Y2, FrameColor%, -1, 1, 1)

     FrameTitleStringLen = LEN(FrameTitle$)
     IF (FrameTitleStringLen > 0) THEN
        CharX = X1 + 16
        CharY = Y1 + 1
        BlankText$ = SPACE$(FrameTitleStringLen + 1)
        CALL EXQPRINTP(BlankText$, (CharX - 4), CharY, FrameColor%, FillColor%)
        CALL EXQPRINTP(FrameTitle$, CharX, CharY, FrameColor%, FillColor%)
     END IF

     CALL MMCURSORON

END SUB

SUB DrawGenericButton (ULC%, ULR%, LRC%, LRR%, State%)
' ---------------------- DrawGenericButton -------------------------------
'
'  DrawGenericButton will draw a GUI Generic button at the
'           passed coordinates with the passed state.
'
'  Calling Convention:
'  DrawGenericButton(ULC%, ULR%, LRC%, LRR%, State%)
'
'  ULC%          Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%          Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  LRC%          Meaningless (The Vertical Scroll Bar width is fixed).
'  LRR%          Bottom right corner Y coordinate.  This parameter will
'                set the Vertical Scroll Bar Height.
'  State%        As follows:
'                       State = 0    Button Normal
'                       State = 1    Button Depressed
'
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     X1 = ULC + 1
     Y1 = ULR + 1
     X2 = LRC - 1
     Y2 = LRR - 1

     CALL BOXDRAWP(X1, Y1, X2, Y2, BLACK, LIGHTGRAY, 1, 1)       ' start drawing the button

     IF (State AND DEPRESSED) THEN
        CALL ELINE(X1, Y1, X2, Y1, DARKGRAY)
        CALL ELINE(X1, Y1, X1, Y2, DARKGRAY)

        X1 = X1 + 1:  Y1 = Y1 + 1
     ELSE
        CALL ELINE(X1, Y1, (X2 - 1), Y1, WHITE)
        CALL ELINE(X1, Y1, X1, (Y2 - 1), WHITE)

        CALL ELINE((X1 + 1), Y2, X2, Y2, DARKGRAY)
        CALL ELINE(X2, (Y1 + 1), X2, Y2, DARKGRAY)

        X1 = X1 + 1:  Y1 = Y1 + 1:  X2 = X2 - 1:  Y2 = Y2 - 1

        CALL ELINE(X1, Y1, (X2 - 1), Y1, WHITE)
        CALL ELINE(X1, Y1, X1, (Y2 - 1), WHITE)

        CALL ELINE((X1 + 1), Y2, X2, Y2, DARKGRAY)
        CALL ELINE(X2, (Y1 + 1), X2, Y2, DARKGRAY)
     END IF

END SUB

SUB DrawGenericDrive (ULC%, ULR%)
' ---------------------- DrawGenericDrive --------------------------------
'
'  DrawGenericDrive will draw a GUI Generic Drive at the
'                   passed coordinates.
'
'  Calling Convention:  DrawGenericDrive(ULC%, ULR%)
'
'  ULC%      Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%      Upper left corner Y coordinate (0 <= ULR <= YMAX)
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     CONST DRIVEWIDTH = 40
     CONST DRIVEHEIGHT = 17

     X1 = ULC + 1
     Y1 = ULR + 1
     X2 = ULC + DRIVEWIDTH - 1 - 1
     Y2 = ULR + DRIVEHEIGHT - 1 - 1

     CALL BOXDRAWP(X1, Y1, X2, Y2, BLACK, LIGHTGRAY, 1, 1)       ' start drawing the Drive

     CALL ELINE(X1, Y1, (X2 - 1), Y1, WHITE)
     CALL ELINE(X1, Y1, X1, (Y2 - 1), WHITE)

     CALL ELINE((X1 + 1), Y2, X2, Y2, DARKGRAY)
     CALL ELINE(X2, (Y1 + 1), X2, Y2, DARKGRAY)

END SUB

SUB DrawHardDrive (ULC%, ULR%, State%, DriveLetter$)
'-------------------------- DrawHardDrive --------------------------------
'
'  DrawHardDrive will draw a GUI Hard Drive at the passed
'                coordinates with the passed State.
'
'  Calling Convention:
'  DrawHardDrive(ULC%, ULR%, State%, DriveLetter$)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                       State = 0    Normal Hard Drive
'                       State = 2    Hard Drive with FOCUS
'  DriveLetter$ Text for the Drive Letter
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     CONST DRIVEWIDTH = 40
     CONST DRIVEHEIGHT = 17

     SELECT CASE GETCHARSIZE                                     ' get font size according to the current CHARSIZE
     CASE 0
        FontHeight = 8
     CASE 1
        FontHeight = 14
     CASE 2
        FontHeight = 16
     CASE ELSE
        EXIT SUB
     END SELECT

     CALL MMCURSOROFF

     TempX1 = ULC + 1                                            ' start drawing the HardDrive
     TempY1 = ULR + 1
     TempX2 = ULC + 69 - 1 - 1
     TempY2 = ULR + 27 - 1 - 1

     IF (State AND FOCUS) THEN
        CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, BLACK, 1, 1)
     ELSE
        CALL BOXDRAWP(TempX1, TempY1, TempX2, TempY2, BLACK, WHITE, 1, 1)
     END IF

     X1 = ULC + 8
     Y1 = ULR + 5
     CALL DrawGenericDrive(X1, Y1)

     TempX1 = X1 + 4
     TempY1 = Y1 + 9
     TempX2 = X1 + DRIVEWIDTH - 6
     CALL ELINE(TempX1, TempY1, TempX2, TempY1, BLACK)
     TempY1 = TempY1 + 2
     CALL ELINE(TempX1, TempY1, TempX2, TempY1, BLACK)
     TempY1 = TempY1 + 2
     TempX2 = TempX1 + 4
     CALL ELINE(TempX1, TempY1, TempX2, TempY1, BLACK)

     TempX1 = ULC + 54
     TempY1 = ULR + ((28 - FontHeight) \ 2)
     TempDriveLetter$ = MID$(DriveLetter$, 1, 1)
     IF (State AND FOCUS) THEN
        CALL EXQPRINTP(TempDriveLetter$, TempX1, TempY1, LIGHTGRAY, BLACK)
     ELSE
        CALL EXQPRINTP(TempDriveLetter$, TempX1, TempY1, BLACK, WHITE)
     END IF

     CALL MMCURSORON

END SUB

SUB DrawHorizontalScrollBar (ULC%, ULR%, LRC%, LRR%, State%, BarPosition%)
' --------------------- DrawHorizontalScrollBar --------------------------
'
'  DrawHorizontalScrollBar will draw a GUI Horizontal Scroll Bar *
'                        at the passed coordinates with the passed
'                        State.
'
'  Calling Convention:
'  DrawHorizontalScrollBar(ULC%, ULR%, LRC%, LRR%, State%, BarPosition%)
'
'   ULC%          Upper left corner X coordinate (0 <= ULC <= XMAX)
'   ULR%          Upper left corner Y coordinate (0 <= ULR <= YMAX)
'   LRC%          Bottom right corner X coordinate.  This parameter will
'                 set the Horizontal Scroll Bar Width.
'   LRR%          Meaningless (The Horizontal Scroll Bar width is fixed)
'   State%        As follows:
'                       State = -1   Only update BarPosition
'                       State = 0    Left and Right Button Normal
'                       State = 1    Left Button Depressed
'                       State = 2    Right Button Depressed
'                       State = 3    Top and Bottom Button Depressed
'   BarPosition%  Relates the Bar Position as a percentage of the
'                 Horizontal Scroll Bar's Width
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     CONST ArrowHeight = 11

     DIM LineMask(1 TO 9) AS INTEGER
     LineMask(1) = &H400
     LineMask(2) = &HE00
     LineMask(3) = &H1F00
     LineMask(4) = &H3F80
     LineMask(5) = &H7FC0
     LineMask(6) = &HFFE0
     LineMask(7) = &H1F00
     LineMask(8) = &H1F00
     LineMask(9) = &H1F00

     IF (ULC < LRC) THEN
        X1 = ULC
        X2 = LRC
     ELSE
        X1 = LRC
        X2 = ULC
     END IF
     Y1 = ULR
     Y2 = ULR + ScrollBarBoxSize - 1

     InteriorScrollBarWidth = (X2 - X1 + 1) - (3 * ScrollBarBoxSize) + 1     ' check ScrollBar for invalid width parameters
     IF (InteriorScrollBarWidth < 0) THEN
        EXIT SUB
     END IF

     IF (BarPosition < 0) OR (BarPosition > 1000) THEN           ' check for invalid BarPosition parameter
        EXIT SUB
     END IF

     CALL MMCURSOROFF

     IF (State <> -1) THEN                                       ' update BarPosition ONLY
        TempX2 = X1 + ScrollBarBoxSize - 1                       ' draw left arrow box
        IF (State AND LEFTDEPRESSED) THEN
           CALL DrawGenericButton(X1, Y1, TempX2, Y2, DEPRESSED)
           TempX1 = X1 + 7
           TempY1 = Y1 + 5
        ELSE
           CALL DrawGenericButton(X1, Y1, TempX2, Y2, NORMAL)
           TempX1 = X1 + 6
           TempY1 = Y1 + 4
        END IF

        TempY2 = TempY1 + ArrowHeight                            ' draw left arrow
        FOR I = 1 TO 9
           CALL SETLINESTYLE(LineMask(I))
           CALL ELINE(TempX1, TempY1, TempX1, TempY2, BLACK)
           TempX1 = TempX1 + 1
        NEXT
        CALL SETLINESTYLE(&HFFFF)

        TempX1 = X2 - ScrollBarBoxSize                           ' draw right arrow box
        IF (State AND RIGHTDEPRESSED) THEN
           CALL DrawGenericButton(TempX1, Y1, X2, Y2, DEPRESSED)
           TempX1 = TempX1 + 8
           TempY1 = Y1 + 5
        ELSE
           CALL DrawGenericButton(TempX1, Y1, X2, Y2, NORMAL)
           TempX1 = TempX1 + 7
           TempY1 = Y1 + 4
        END IF

        TempY2 = TempY1 + ArrowHeight                            ' draw right arrow
        FOR I = 9 TO 1 STEP -1
           CALL SETLINESTYLE(LineMask(I))
           CALL ELINE(TempX1, TempY1, TempX1, TempY2, BLACK)
           TempX1 = TempX1 + 1
        NEXT
        CALL SETLINESTYLE(&HFFFF)
     END IF

     TempInteriorScrollBarWidth& = InteriorScrollBarWidth        ' draw BarPosition box
     TempBarPosition& = BarPosition
     TempVar& = (TempInteriorScrollBarWidth& * TempBarPosition&) \ 1000&
     TempX1 = X1 + ScrollBarBoxSize - 1 + TempVar&
     TempX2 = TempX1 + ScrollBarBoxSize - 1

     CALL BOXDRAWP((X1 + ScrollBarBoxSize), (Y1 + 1), (X2 - ScrollBarBoxSize - 1), (Y2 - 1), BLACK, LIGHTGRAY, 1, 1)' start drawing the HorizontalScrollBar

     CALL DrawGenericButton(TempX1, Y1, TempX2, Y2, NORMAL)

     CALL MMCURSORON

END SUB

SUB DrawLeftArrowBut (ULC%, ULR%, State%)
'------------------------ DrawLeftArrowBut -------------------------------
'
'  DrawLeftArrowBut will draw a GUI Left Arrow Button at the
'                   passed coordinates with the passed State.
'
'  Calling Convention: DrawLeftArrowBut(ULC%, ULR%, State%)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                       State = 0    Normal Left Arrow Button
'                       State = 1    Depressed Left Arrow Button
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     DIM LineMask(1 TO 4) AS INTEGER

     CALL GetVideoMode(CurrentVideoMode)                         ' set LeftArrow button size according to the current video mode
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        BoxHeight = 11
        BoxWidth = 12
        XIndent = 3
        YIndent = 3

        LineMask(1) = &H2000
        LineMask(2) = &H7000
        LineMask(3) = &HF800
        LineMask(4) = &H0

        ArrowHeight = 5

     CASE 18
        BoxHeight = 11
        BoxWidth = 10
        XIndent = 2
        YIndent = 3

        LineMask(1) = &H2000
        LineMask(2) = &H7000
        LineMask(3) = &HF800
        LineMask(4) = &H0

        ArrowHeight = 5

     CASE 91
        BoxHeight = 11
        BoxWidth = 14
        XIndent = 3
        YIndent = 2

        LineMask(1) = &H1000
        LineMask(2) = &H3800
        LineMask(3) = &H7C00
        LineMask(4) = &HFE00

        ArrowHeight = 7

     CASE ELSE
        EXIT SUB
     END SELECT

     CALL MMCURSOROFF

     CALL BOXDRAWP((ULC + 1), (ULR + 1), (ULC + BoxWidth - 2), (ULR + BoxHeight - 2), BLACK, LIGHTGRAY, 1, 1)' start drawing the LeftArrow
     IF (State AND DEPRESSED) THEN
        ArrowColor = WHITE
     ELSE
        ArrowColor = BLACK
     END IF

     TempX = ULC + XIndent
     TempY1 = ULR + YIndent
     TempY2 = TempY1 + ArrowHeight - 1
     FOR I = 1 TO 4
        IF (LineMask(I)) THEN
             CALL SETLINESTYLE(LineMask(I))
             CALL ELINE(TempX, TempY1, TempX, TempY2, ArrowColor)
             TempX = TempX + 1
             CALL ELINE(TempX, TempY1, TempX, TempY2, ArrowColor)
             TempX = TempX + 1
        END IF
     NEXT
     CALL SETLINESTYLE(&HFFFF)

     CALL MMCURSORON

END SUB

SUB DrawMaxBut (ULC%, ULR%, State%)
'--------------------------- DrawMaxBut ----------------------------------
'
'  DrawMaxBut will draw a GUI Maximum Button at the passed
'             coordinates with the passed State.
'
'  Calling Convention: DrawMaxBut(ULC%, ULR%, State%)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                       State = 0    Normal Max Button
'                       State = 1    Depressed Max Button
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     DIM LineMask(1 TO 6) AS INTEGER

     SELECT CASE GETCHARSIZE                                     ' set MaxBut size according to the current CHARSIZE
     CASE 0
        BoxHeight = 16
        YIndent = 6

        LineMask(1) = &H1000
        LineMask(2) = &H3800
        LineMask(3) = &H7C00
        LineMask(4) = &HFE00
        LineMask(5) = &H0
        LineMask(6) = &H0

        ButtonWidth = 7

     CASE 1
        BoxHeight = 20
        YIndent = 7

        LineMask(1) = &H800
        LineMask(2) = &H1C00
        LineMask(3) = &H3E00
        LineMask(4) = &H7F00
        LineMask(5) = &HFF80
        LineMask(6) = &H0

        ButtonWidth = 9

     CASE 2
        BoxHeight = 22
        YIndent = 8

        LineMask(1) = &H400
        LineMask(2) = &HE00
        LineMask(3) = &H1F00
        LineMask(4) = &H3F80
        LineMask(5) = &H7FC0
        LineMask(6) = &HFFE0

        ButtonWidth = 11
     CASE ELSE
        EXIT SUB
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        BoxWidth = BoxHeight + 3
     CASE 18
        BoxWidth = BoxHeight + 1
     CASE 91
        BoxWidth = BoxHeight + 1
     CASE ELSE
        EXIT SUB
     END SELECT
     XIndent = (BoxWidth - ButtonWidth) \ 2

     CALL MMCURSOROFF

     IF (State AND DEPRESSED) THEN                               ' start drawing the MaxBut
        CALL DrawGenericButton(ULC, ULR, (ULC + BoxWidth - 1), (ULR + BoxHeight - 1), DEPRESSED)

        TempX1 = ULC + XIndent + 1
        TempY = ULR + YIndent + 1
        TempX2 = TempX1 + ButtonWidth
        FOR I = 1 TO 6
           CALL SETLINESTYLE(LineMask(I))
           CALL ELINE(TempX1, TempY, TempX2, TempY, BLACK)
           TempY = TempY + 1
        NEXT
        CALL SETLINESTYLE(&HFFFF)
     ELSE
        CALL DrawGenericButton(ULC, ULR, (ULC + BoxWidth - 1), (ULR + BoxHeight - 1), NORMAL)

        TempX1 = ULC + XIndent
        TempY = ULR + YIndent
        TempX2 = TempX1 + ButtonWidth
        FOR I = 1 TO 6
           CALL SETLINESTYLE(LineMask(I))
           CALL ELINE(TempX1, TempY, TempX2, TempY, BLACK)
           TempY = TempY + 1
        NEXT
        CALL SETLINESTYLE(&HFFFF)
     END IF

     CALL MMCURSORON

END SUB

SUB DrawMinBut (ULC%, ULR%, State%)
' -------------------------- DrawMinBut ----------------------------------
'
'  DrawMinBut will draw a GUI Minimum Button at the passed
'             coordinates with the passed State.
'
'  Calling Convention: DrawMinBut(ULC%, ULR%, State%)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                       State = 0    Normal Min Button
'                       State = 1    Depressed Min Button
'
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     DIM LineMask(1 TO 6) AS INTEGER

     SELECT CASE GETCHARSIZE                                     ' set MinBut size according to the current CHARSIZE
     CASE 0
        BoxHeight = 16
        YIndent = 7

        LineMask(1) = &HFE00
        LineMask(2) = &H7C00
        LineMask(3) = &H3800
        LineMask(4) = &H1000
        LineMask(5) = &H0
        LineMask(6) = &H0

        ButtonWidth = 7

     CASE 1
        BoxHeight = 20
        YIndent = 8

        LineMask(1) = &HFF80
        LineMask(2) = &H7F00
        LineMask(3) = &H3E00
        LineMask(4) = &H1C00
        LineMask(5) = &H800
        LineMask(6) = &H0

        ButtonWidth = 9

     CASE 2
        BoxHeight = 22
        YIndent = 9

        LineMask(1) = &HFFE0
        LineMask(2) = &H7FC0
        LineMask(3) = &H3F80
        LineMask(4) = &H1F00
        LineMask(5) = &HE00
        LineMask(6) = &H400

        ButtonWidth = 11
     CASE ELSE
        EXIT SUB
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        BoxWidth = BoxHeight + 3
     CASE 18
        BoxWidth = BoxHeight + 1
     CASE 91
        BoxWidth = BoxHeight + 1
     CASE ELSE
        EXIT SUB
     END SELECT
     XIndent = (BoxWidth - ButtonWidth) \ 2

     CALL MMCURSOROFF

     IF (State AND DEPRESSED) THEN                               ' start drawing the MinBut
        CALL DrawGenericButton(ULC, ULR, (ULC + BoxWidth - 1), (ULR + BoxHeight - 1), DEPRESSED)

        TempX1 = ULC + XIndent + 1
        TempY = ULR + YIndent + 1
        TempX2 = TempX1 + ButtonWidth
        FOR I = 1 TO 6
           CALL SETLINESTYLE(LineMask(I))
           CALL ELINE(TempX1, TempY, TempX2, TempY, BLACK)
           TempY = TempY + 1
        NEXT
        CALL SETLINESTYLE(&HFFFF)
     ELSE
        CALL DrawGenericButton(ULC, ULR, (ULC + BoxWidth - 1), (ULR + BoxHeight - 1), NORMAL)

        TempX1 = ULC + XIndent
        TempY = ULR + YIndent
        TempX2 = TempX1 + ButtonWidth
        FOR I = 1 TO 6
           CALL SETLINESTYLE(LineMask(I))
           CALL ELINE(TempX1, TempY, TempX2, TempY, BLACK)
           TempY = TempY + 1
        NEXT
        CALL SETLINESTYLE(&HFFFF)
     END IF

     CALL MMCURSORON

END SUB

SUB DrawMinMaxBut (ULC%, ULR%, State%)
'------------------------ DrawMinMaxBut ----------------------------------
'
'  DrawMinMaxBut will draw a GUI Minimum/Maximum Button at the
'                passed coordinates with the passed State.
'
'  Calling Convention: DrawMinMaxBut(ULC%, ULR%, State%)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                       State = 0    Normal MinMax Button
'                       State = 1    Depressed MinMax Button
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     DIM LineMask(1 TO 11) AS INTEGER

     SELECT CASE GETCHARSIZE                                     ' set MinMax button size according to the current CHARSIZE
     CASE 0
        BoxHeight = 16
        YIndent = 5

        LineMask(1) = &H2000
        LineMask(2) = &H7000
        LineMask(3) = &HF800
        LineMask(4) = &H0
        LineMask(5) = &HF800
        LineMask(6) = &H7000
        LineMask(7) = &H2000
        LineMask(8) = &H0
        LineMask(9) = &H0
        LineMask(10) = &H0
        LineMask(11) = &H0

        ButtonWidth = 5

     CASE 1
        BoxHeight = 20
        YIndent = 6

        LineMask(1) = &H1000
        LineMask(2) = &H3800
        LineMask(3) = &H7C00
        LineMask(4) = &HFE00
        LineMask(5) = &H0
        LineMask(6) = &HFE00
        LineMask(7) = &H7C00
        LineMask(8) = &H3800
        LineMask(9) = &H1000
        LineMask(10) = &H0
        LineMask(11) = &H0

        ButtonWidth = 7

     CASE 2
        BoxHeight = 22
        YIndent = 6

        LineMask(1) = &H800
        LineMask(2) = &H1C00
        LineMask(3) = &H3E00
        LineMask(4) = &H7F00
        LineMask(5) = &HFF80
        LineMask(6) = &H0
        LineMask(7) = &HFF80
        LineMask(8) = &H7F00
        LineMask(9) = &H3E00
        LineMask(10) = &H1C00
        LineMask(11) = &H800

        ButtonWidth = 9

     CASE ELSE
        EXIT SUB
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        BoxWidth = BoxHeight + 3
     CASE 18
        BoxWidth = BoxHeight + 1
     CASE 91
        BoxWidth = BoxHeight + 1
     CASE ELSE
        EXIT SUB
     END SELECT
     XIndent = (BoxWidth - ButtonWidth) \ 2

     CALL MMCURSOROFF

     IF (State AND DEPRESSED) THEN                               ' start drawing the MinMaxBut
        CALL DrawGenericButton(ULC, ULR, (ULC + BoxWidth - 1), (ULR + BoxHeight - 1), DEPRESSED)

        TempX1 = ULC + XIndent + 1
        TempY = ULR + YIndent + 1
        TempX2 = TempX1 + ButtonWidth
        FOR I = 1 TO 11
           IF (LineMask(I)) THEN
             CALL SETLINESTYLE(LineMask(I))
             CALL ELINE(TempX1, TempY, TempX2, TempY, BLACK)
           END IF
           TempY = TempY + 1
        NEXT
        CALL SETLINESTYLE(&HFFFF)
     ELSE
        CALL DrawGenericButton(ULC, ULR, (ULC + BoxWidth - 1), (ULR + BoxHeight - 1), NORMAL)

        TempX1 = ULC + XIndent
        TempY = ULR + YIndent
        TempX2 = TempX1 + ButtonWidth
        FOR I = 1 TO 11
           IF (LineMask(I)) THEN
             CALL SETLINESTYLE(LineMask(I))
             CALL ELINE(TempX1, TempY, TempX2, TempY, BLACK)
           END IF
           TempY = TempY + 1
        NEXT
        CALL SETLINESTYLE(&HFFFF)
     END IF

     CALL MMCURSORON

END SUB

SUB DrawOptionBut (ULC%, ULR%, State%, Caption$, Fore%, Back%)
' -------------------------- DrawOptionBut --------------------------------
'
'  DrawOptionBut will draw a GUI Option Button (Radio Button)
'                at the passed coordinates with the passed State and
'                Caption.
'
'  Calling Convention:
'  DrawOptionBut(ULC%, ULR%, State%, Caption$, Fore%, Back%)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                                    Non-Selected (Empty Check Box)
'                       State = 0    Normal State
'                       State = 1    Command Button Depressed
'                       State = 2    Command Button with Focus
'                       State = 3    Commnad Button Depressed with Focus *
'                                    Selected (X in Check Box)
'                       State = 4    Normal State
'                       State = 5    Command Button Depressed
'                       State = 6    Command Button with Focus
'                       State = 7    Commnad Button Depressed with Focus *
'  Caption$     Text for Check Box
'  Fore%        Foreground Color (0 <= Fore% <= 15)
'  Back%        Background Color (0 <= Back% <= 15)
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SELECT CASE GETCHARSIZE                                     ' set font height according to CHARSIZE
     CASE 0
        FontHeight = 8
        OptionButRadius = 5
        InnerCircleRadius = 2
        CharX = 16
        CharY = 3
     CASE 1
        FontHeight = 14
        OptionButRadius = 6
        InnerCircleRadius = 3
        CharX = 21
        CharY = 1
     CASE 2
        FontHeight = 16
        OptionButRadius = 7
        InnerCircleRadius = 4
        CharX = 23
        CharY = 2
     CASE ELSE
        EXIT SUB
     END SELECT

     CALL MMCURSOROFF

     CaptionLen = 8 * LEN(Caption$)

     TempX1 = ULC + OptionButRadius                              ' start drawing the OptionBut
     TempY1 = ULR + OptionButRadius

     IF (State AND DEPRESSED) THEN
        CALL EFILLEDCIRCLE(TempX1, TempY1, OptionButRadius, Fore%)
        CALL EFILLEDCIRCLE(TempX1, TempY1, (OptionButRadius - 2), Back%)
     ELSE
        CALL EFILLEDCIRCLE(TempX1, TempY1, OptionButRadius, Back%)
        CALL ECIRCLE(TempX1, TempY1, OptionButRadius, Fore%)
     END IF

     IF (State AND SELECTED) THEN
        CALL EFILLEDCIRCLE(TempX1, TempY1, InnerCircleRadius, Fore%)
     END IF

     CharX = ULC + CharX
     CharY = ULR + CharY
     CALL EXQPRINTP(Caption$, CharX, CharY, Fore%, Back%)

     IF (State AND FOCUS) THEN
        CALL FocusBox((CharX - 3), (CharY - 2), (CharX + CaptionLen + 3), (CharY + FontHeight), Fore%)
     ELSE
        CALL FocusBox((CharX - 3), (CharY - 2), (CharX + CaptionLen + 3), (CharY + FontHeight), Back%)
     END IF

     CALL MMCURSORON

END SUB

SUB DrawRightArrowBut (ULC%, ULR%, State%)
'----------------------- DrawRightArrowBut -------------------------------
'
'  DrawRightArrowBut will draw a GUI Right Arrow Button at the
'                    passed coordinates with the passed State.
'
'  Calling Convention: DrawRightArrowBut(ULC%, ULR%, State%)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                       State = 0    Normal Right Arrow Button
'                       State = 1    Depressed Right Arrow Button
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     DIM LineMask(1 TO 4) AS INTEGER

     CALL GetVideoMode(CurrentVideoMode)                         ' set RightArrow button size according to the current video mode
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        BoxHeight = 11
        BoxWidth = 12
        XIndent = 3
        YIndent = 3

        LineMask(1) = &HF800
        LineMask(2) = &H7000
        LineMask(3) = &H2000
        LineMask(4) = &H0

        ArrowHeight = 5

     CASE 18
        BoxHeight = 11
        BoxWidth = 10
        XIndent = 2
        YIndent = 3

        LineMask(1) = &HF800
        LineMask(2) = &H7000
        LineMask(3) = &H2000
        LineMask(4) = &H0

        ArrowHeight = 5

     CASE 91
        BoxHeight = 11
        BoxWidth = 14
        XIndent = 3
        YIndent = 2

        LineMask(1) = &HFE00
        LineMask(2) = &H7C00
        LineMask(3) = &H3800
        LineMask(4) = &H1000

        ArrowHeight = 7

     CASE ELSE
        EXIT SUB
     END SELECT

     CALL MMCURSOROFF

     CALL BOXDRAWP((ULC + 1), (ULR + 1), (ULC + BoxWidth - 2), (ULR + BoxHeight - 2), BLACK, LIGHTGRAY, 1, 1)' start drawing the RightArrow
     IF (State AND DEPRESSED) THEN
        ArrowColor = WHITE
     ELSE
        ArrowColor = BLACK
     END IF

     TempX = ULC + XIndent
     TempY1 = ULR + YIndent
     TempY2 = TempY1 + ArrowHeight - 1
     FOR I = 1 TO 4
        IF (LineMask(I)) THEN
             CALL SETLINESTYLE(LineMask(I))
             CALL ELINE(TempX, TempY1, TempX, TempY2, ArrowColor)
             TempX = TempX + 1
             CALL ELINE(TempX, TempY1, TempX, TempY2, ArrowColor)
             TempX = TempX + 1
        END IF
     NEXT
     CALL SETLINESTYLE(&HFFFF)

     CALL MMCURSORON

END SUB

SUB DrawTextBox (ULC%, ULR%, LRC%, LRR%, State%, Justify%, Caption$, HotColor%, Fore%, Back%)
'-------------------------- DrawTextBox -------------------------------
'
'  DrawTextBox will draw a GUI Text Box at the passed
'              coordinates with the passed State, Caption, and colors.
'
'  Calling Convention:
'  DrawTextBox(ULC%, ULR%, LRC%, LRR%, State%, Justify%, Caption$, HotColor%, Fore%, Back%)
'

'   ULC%        Upper left corner X coordinate (0 <= ULC <= XMAX)
'   ULR%        Upper left corner Y coordinate (0 <= ULR <= YMAX)
'   LRC%        Bottom right corner X coordinate (0 <= LRC <= XMAX).
'               If LRC = -1, then DrawText will calculate the
'                                appropriate coordinate based on the
'                                string length.
'   LRR%        Bottom right corner Y coordinate (0 <= LRR <= YMAX).
'               If LRR = -1, then DrawText will calculate the
'                                appropriate coordinate based on the
'                                Font Height.
'   State%      As follows:
'                       State = 0    Normal Text Box
'                       State = 2    Text Box with Focus
'   Justify%    If LRC is valid (LRC <> -1) then the text will
'               be justified accordingly:
'                       Justify = 0   Left Justify
'                       Justify = 1   Center Justify
'                       Justify = 2   Right Justify
'   Caption$    Text to be placed in the Text Box
'   HotColor%   The foreground color for and option HOT letter in the
'               Caption.  If Caption has an "&", the following character *
'               is the HOT letter.
'   Fore%       Foreground Color (0 <= Fore% <= 15)
'   Back%       Background Color (0 <= Back% <= 15)
'
' returned modified       :
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     IF (Justify% < LEFTJUSTIFY) OR (Justify% > RIGHTJUSTIFY) THEN     ' check for invalid Justify%
        EXIT SUB
     END IF

     SELECT CASE GETCHARSIZE                                     ' set font height
     CASE 0
        FontHeight = 8
        YIndent = 1
     CASE 1
        FontHeight = 14
        YIndent = 1
     CASE 2
        FontHeight = 16
        YIndent = 2
     CASE ELSE
        EXIT SUB
     END SELECT

     CALL MMCURSOROFF

     HotKeyLocation = INSTR(Caption$, "&")                       ' check for hot key
     IF (HotKeyLocation) THEN
        HotKeyLetter$ = MID$(Caption$, (HotKeyLocation + 1), 1)
        TempCaption$ = LEFT$(Caption$, (HotKeyLocation - 1)) + MID$(Caption$, (HotKeyLocation + 1))
     ELSE
        TempCaption$ = Caption$
     END IF
     CaptionStringLen = LEN(TempCaption$)
     CaptionPixelLen = 8 * CaptionStringLen

     X1 = ULC                                                    ' set TextBox coordinates
     IF (LRC = -1) THEN
        X2 = X1 + CaptionPixelLen + 8 - 1
     ELSE
        IF (ULC < LRC) THEN
             X2 = LRC
        ELSE
             X1 = LRC
             X2 = ULC
        END IF
     END IF

     Y1 = ULR
     IF (LRR = -1) THEN
        Y2 = Y1 + FontHeight + 4 - 1
     ELSE
        IF (ULR < LRR) THEN
             Y2 = LRR
        ELSE
             Y1 = LRR
             Y2 = ULR
        END IF
     END IF

     TempBack = Back%                                            ' draw background
     IF (State AND FOCUS) THEN
        TempBack = Fore%
     END IF
     CALL BOXDRAWP(X1, Y1, X2, Y2, TempBack, TempBack, 0, 0)

     CharY = Y1 + ((Y2 - Y1 - FontHeight - 1) \ 2) + YIndent      ' place the Text

     SELECT CASE Justify%
     CASE LEFTJUSTIFY                                            '  Left Justify
        CharX = X1 + 4

     CASE CENTERJUSTIFY                                          '  Center Justify
        CharX = X1 + ((X2 - X1 - CaptionPixelLen) \ 2)

     CASE RIGHTJUSTIFY                                           '  Right Justify
        CharX = X2 - CaptionPixelLen - 4
     END SELECT

     IF (State AND FOCUS) THEN
        CALL EXQPRINTP(TempCaption$, CharX, CharY, Back%, Fore%)
     ELSE
        CALL EXQPRINTP(TempCaption$, CharX, CharY, Fore%, Back%)
     END IF

     IF (HotKeyLocation) THEN
        CharX = CharX + 8 * (HotKeyLocation - 1)
        CALL EXQPRINTP(HotKeyLetter$, CharX, CharY, HotColor%, TempBack)
     END IF

     CALL MMCURSORON

END SUB

SUB DrawUpArrowBut (ULC%, ULR%, State%)
'-------------------------- DrawUpArrowBut -------------------------------
'
'  DrawUpArrowBut will draw a GUI Up Arrow Button at the passed
'                 coordinates with the passed State.
'
'  Calling Convention: DrawUpArrowBut(ULC%, ULR%, State%)
'
'  ULC%         Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%         Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  State%       As follows:
'                       State = 0    Normal Up Arrow Button
'                       State = 1    Depressed Up Arrow Button
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     DIM LineMask(1 TO 5) AS INTEGER

     CALL GetVideoMode(CurrentVideoMode)                         ' set UpArrow button size according to the current video mode
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        BoxHeight = 10
        BoxWidth = 13
        XIndent = 3
        YIndent = 3

        LineMask(1) = &H1000
        LineMask(2) = &H3800
        LineMask(3) = &H7C00
        LineMask(4) = &HFE00
        LineMask(5) = &H0

        ArrowWidth = 7

     CASE 18
        BoxHeight = 10
        BoxWidth = 11
        XIndent = 2
        YIndent = 3

        LineMask(1) = &H1000
        LineMask(2) = &H3800
        LineMask(3) = &H7C00
        LineMask(4) = &HFE00
        LineMask(5) = &H0

        ArrowWidth = 7

     CASE 91
        BoxHeight = 11
        BoxWidth = 15
        XIndent = 3
        YIndent = 3

        LineMask(1) = &H800
        LineMask(2) = &H1C00
        LineMask(3) = &H3E00
        LineMask(4) = &H7F00
        LineMask(5) = &HFF80

        ArrowWidth = 9

     CASE ELSE
        EXIT SUB
     END SELECT

     CALL MMCURSOROFF

     CALL BOXDRAWP((ULC + 1), (ULR + 1), (ULC + BoxWidth - 2), (ULR + BoxHeight - 2), BLACK, LIGHTGRAY, 1, 1)' start drawing the UpArrow
     IF (State AND DEPRESSED) THEN
        ArrowColor = WHITE
     ELSE
        ArrowColor = BLACK
     END IF

     TempX1 = ULC + XIndent
     TempY = ULR + YIndent
     TempX2 = TempX1 + ArrowWidth - 1
     FOR I = 1 TO 5
        CALL SETLINESTYLE(LineMask(I))
        CALL ELINE(TempX1, TempY, TempX2, TempY, ArrowColor)
        TempY = TempY + 1
     NEXT
     CALL SETLINESTYLE(&HFFFF)

     CALL MMCURSORON

END SUB

SUB DrawVerticalScrollBar (ULC%, ULR%, LRC%, LRR%, State%, BarPosition%)
' ------------------ DrawVerticalScrollBar -------------------------------
'
'  DrawVerticalScrollBar will draw a GUI Vertical Scroll Bar
'                        at the passed coordinates with the passed
'                        State.
'
'  Calling Convention:
'  DrawVerticalScrollBar(ULC%, ULR%, LRC%, LRR%, State%, BarPosition%)
'
'  ULC%          Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%          Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  LRC%          Meaningless (The Vertical Scroll Bar width is fixed).
'  LRR%          Bottom right corner Y coordinate.  This parameter will
'                set the Vertical Scroll Bar Height.
'  State%        As follows:
'                       State = -1   Only update BarPosition
'                       State = 0    Top and Bottom Button Normal
'                       State = 1    Top Button Depressed
'                       State = 2    Bottom Button Depressed
'                       State = 3    Top and Bottom Button Depressed
'  BarPosition%  Relates the Bar Position as a percentage of the
'                Vertical Scroll Bar's Height
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     CONST ArrowWidth = 11

     DIM LineMask(1 TO 9) AS INTEGER
     LineMask(1) = &H400
     LineMask(2) = &HE00
     LineMask(3) = &H1F00
     LineMask(4) = &H3F80
     LineMask(5) = &H7FC0
     LineMask(6) = &HFFE0
     LineMask(7) = &H1F00
     LineMask(8) = &H1F00
     LineMask(9) = &H1F00

     IF (ULR < LRR) THEN
        Y1 = ULR
        Y2 = LRR
     ELSE
        Y1 = LRR
        Y2 = ULR
     END IF
     X1 = ULC
     X2 = ULC + ScrollBarBoxSize - 1

     InteriorScrollBarHeight = (Y2 - Y1 + 1) - (3 * ScrollBarBoxSize) + 1     ' check ScrollBar for invalid height parameters
     IF (InteriorScrollBarHeight < 0) THEN
        EXIT SUB
     END IF

     IF (BarPosition < 0) OR (BarPosition > 1000) THEN           ' check for invalid BarPosition parameter
        EXIT SUB
     END IF

     CALL MMCURSOROFF

     IF (State <> -1) THEN                                       ' update BarPosition ONLY
        TempX2 = X1 + ScrollBarBoxSize - 1                       ' draw top arrow box
        TempY2 = Y1 + ScrollBarBoxSize - 1
        IF (State AND TOPDEPRESSED) THEN
           CALL DrawGenericButton(X1, Y1, TempX2, TempY2, DEPRESSED)
           TempX1 = X1 + 6
           TempY1 = Y1 + 7
        ELSE
           CALL DrawGenericButton(X1, Y1, TempX2, TempY2, NORMAL)
           TempX1 = X1 + 5
           TempY1 = Y1 + 6
        END IF

        TempX2 = TempX1 + ArrowWidth                             ' draw top arrow
        FOR I = 1 TO 9
           CALL SETLINESTYLE(LineMask(I))
           CALL ELINE(TempX1, TempY1, TempX2, TempY1, BLACK)
           TempY1 = TempY1 + 1
        NEXT
        CALL SETLINESTYLE(&HFFFF)

        TempY1 = Y2 - ScrollBarBoxSize                           ' draw bottom arrow box
           TempX2 = X2
        TempY2 = Y2
        IF (State AND BOTTOMDEPRESSED) THEN
           CALL DrawGenericButton(X1, TempY1, TempX2, TempY2, DEPRESSED)
           TempX1 = X1 + 6
           TempY1 = TempY1 + 8
        ELSE
           CALL DrawGenericButton(X1, TempY1, TempX2, TempY2, NORMAL)
           TempX1 = X1 + 5
           TempY1 = TempY1 + 7
        END IF

        TempX2 = TempX1 + ArrowWidth                             ' draw bottom arrow
        FOR I = 9 TO 1 STEP -1
           CALL SETLINESTYLE(LineMask(I))
           CALL ELINE(TempX1, TempY1, TempX2, TempY1, BLACK)
           TempY1 = TempY1 + 1
        NEXT
        CALL SETLINESTYLE(&HFFFF)
     END IF

     TempInteriorScrollBarHeight& = InteriorScrollBarHeight      ' draw BarPosition box
     TempBarPosition& = BarPosition
     TempVar& = (TempInteriorScrollBarHeight& * TempBarPosition&) \ 1000&
     TempY1 = Y1 + ScrollBarBoxSize - 1 + TempVar&
     TempX2 = X1 + ScrollBarBoxSize - 1
     TempY2 = TempY1 + ScrollBarBoxSize - 1

     CALL BOXDRAWP((X1 + 1), (Y1 + ScrollBarBoxSize), (X2 - 1), (Y2 - ScrollBarBoxSize - 1), BLACK, LIGHTGRAY, 1, 1)' start drawing the VerticalScrollBar

     CALL DrawGenericButton(X1, TempY1, TempX2, TempY2, NORMAL)

     CALL MMCURSORON

END SUB

SUB FlushGUIEvent
'----------------------- FlushGUIEvent --------------------------------
'
' FlushGUIEvent will flush all GUI events.  This is useful when
'               changing object arrays.
'
' Calling Convention:
'    CALL FlushGUIEvent
'
' returned modified       :
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments

      DO
        DONE = TRUE
        CALL MMBUTTON(LeftDepressButton, RightDepressButton)
        IF LeftDepressButton THEN
           DONE = FALSE
        END IF

        CALL MMClick(LeftClickButton, RightClickButton)
        IF LeftClickButton THEN
           DONE = FALSE
        END IF

        CALL KEYPRESS(KeyboardHit)
        IF KeyboardHit THEN
           Junk$ = INKEY$
           DONE = FALSE
        END IF
     LOOP UNTIL DONE
END SUB

SUB FocusBox (ULC%, ULR%, LRC%, LRR%, ForeColor%)
' --------------------------- FocusBox -----------------------------------
'
'  FocusBox will draw a GUI focus box (dotted-line box) at the
'           passed coordinates with the passed color
'
'  Calling Convention:
'  FocusBox(ULC%, ULR%, LRC%, LRR%, ForeColor%)
'
'  ULC%     Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%     Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  LRC%     Bottom right corner X coordinate (0 <= LRC <= XMAX)
'  LRR%     Bottom right corner Y coordinate (0 <= LRR <= YMAX)
'  ForeColor%    Color of the box    (0 <= ForColor% <= 15)
'
'
'  returned modified       :
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     CALL SETLINESTYLE(&H5555)

     CALL ELINE(ULC%, ULR%, LRC%, ULR%, ForeColor%)
     CALL ELINE(ULC%, LRR%, LRC%, LRR%, ForeColor%)
     CALL ELINE(ULC%, ULR%, ULC%, LRR%, ForeColor%)
     CALL ELINE(LRC%, ULR%, LRC%, LRR%, ForeColor%)

     CALL SETLINESTYLE(&HFFFF)

END SUB

FUNCTION GetHandle% (TempObjectArray() AS GUIObject)

     '     (IMPORTANT NOTE:  The ZERO array element is different from the
     '                       rest of the array)
     TempArrayElements = TempObjectArray(0).CurrentUse           ' get total number of array elements

     FOR I = 1 TO TempArrayElements                              ' find first free array element
        IF (TempObjectArray(I).CurrentUse = 0) THEN
             GetHandle% = I
             EXIT FUNCTION
        END IF
     NEXT

     GetHandle% = 0                                              ' set error return code

END FUNCTION

SUB InitObjectArray (ObjectCount%, ObjectArray() AS GUIObject)
'---------------------------- InitObjectArray -------------------------
'
' InitObjectArray will initialize a GUIObject Array.
'
' Calling Convention:
'         InitObjectArray(ObjectCount%, ObjectArray() AS GUIObject)
'
' ObjectCount%    Total number of objects to be used in this Form.
' ObjectArray()   Array of GUIObject TYPEs.
'
' returned modified       :
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     IF (ObjectCount% < 0) THEN                                  ' check for invalid ObjectCount%
        EXIT SUB
     END IF

     '  IMPORTANT NOTE:  Object Array Element ZERO has a specicial significance
     '                   which differentiates it from the rest of the array.
     ObjectArray(0).CurrentUse = ObjectCount%

     FOR I = 1 TO ObjectCount%                                   ' init CurrentUse variable
        ObjectArray(I).CurrentUse = 0
        ObjectArray(I).Enabled = FALSE
     NEXT I

END SUB

FUNCTION SetCheckBox% (ULC%, ULR%, State%, Caption$, Fore%, Back%, ObjectArray() AS GUIObject)

     SetCheckBox% = -1                                           ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     SELECT CASE GETCHARSIZE                                     ' set font height according to the current video mode
     CASE 0
        FontHeight = 8
        CharX = 16
     CASE 1
        FontHeight = 14
        CharX = 22
     CASE 2
        FontHeight = 16
        CharX = 24
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     Y2 = ULR% + FontHeight + 3 - 1                              ' set the CheckBox coordinates
     X2 = ULC% + CharX + 8 * LEN(Caption$) + 7 - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = CheckBoxType         ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = Fore%                      ' set Foreground Color
     ObjectArray(ObjectHandle).Back = Back%                      ' set Background Color
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = LEN(Caption$)        ' set Text String Length
     ObjectArray(ObjectHandle).Text = Caption$                   ' set Text

     SetCheckBox% = ObjectHandle

END FUNCTION

FUNCTION SetComboBox% (ULC%, ULR%, State%, LongestStringLen%, Caption$, ObjectArray() AS GUIObject)

     SetComboBox% = -1                                           ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     SELECT CASE GETCHARSIZE                                     ' set ComboBox button size according to the current CHARSIZE
     CASE 0
        BoxHeight = 16
     CASE 1
        BoxHeight = 20
     CASE 2
        BoxHeight = 22
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        BoxWidth = BoxHeight + 3
     CASE 18
        BoxWidth = BoxHeight + 1
     CASE 91
        BoxWidth = BoxHeight + 1
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     X2 = ULC% + 8 * (LongestStringLen% + 2) + (BoxWidth \ 2) + BoxWidth - 1      ' set the ComboBox
     Y2 = ULR% + BoxHeight - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = ComboBoxType         ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = -1                         ' unused
     ObjectArray(ObjectHandle).Back = -1                         ' unused
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = DirType%          ' unused
     ObjectArray(ObjectHandle).LongestStringLen = LongestStringLen     ' set Longest String Length
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = LEN(Caption)         ' set Text String Len
     ObjectArray(ObjectHandle).Text = Caption$                   ' set Text

     SetComboBox% = ObjectHandle

END FUNCTION

FUNCTION SetCommandBut% (ULC%, ULR%, State%, Caption$, ObjectArray() AS GUIObject)

     SetCommandBut% = -1                                         ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     SELECT CASE GETCHARSIZE                                     ' set font height according to the current video mode
     CASE 0
        FontHeight = 8
     CASE 1
        FontHeight = 14
     CASE 2
        FontHeight = 16
     CASE ELSE
        EXIT FUNCTION
     END SELECT


     X2 = ULC% + 8 * LEN(Caption$) + 40 - 1                      ' set the CommandButton coordinates
     Y2 = ULR% + FontHeight + 18 - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = CommandButType       ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = -1                         ' unused
     ObjectArray(ObjectHandle).Back = -1                         ' unused
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = LEN(Caption$)        ' set Text String Length
     ObjectArray(ObjectHandle).Text = Caption$                   ' set Text

     SetCommandBut% = ObjectHandle

END FUNCTION

FUNCTION SetControlBox% (ULC%, ULR%, State%, ObjectArray() AS GUIObject)

     SetControlBox% = -1                                         ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     SELECT CASE GETCHARSIZE                                     ' set ControlBox size according to the current CHARSIZE
     CASE 0
        BoxHeight = 16
     CASE 1
        BoxHeight = 20
     CASE 2
        BoxHeight = 22
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        BoxWidth = BoxHeight + 3
     CASE 18
        BoxWidth = BoxHeight + 1
     CASE 91
        BoxWidth = BoxHeight + 1
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     X2 = ULC% + BoxWidth - 1
     Y2 = ULR% + BoxHeight - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = ControlBoxType       ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = -1                         ' unused
     ObjectArray(ObjectHandle).Back = -1                         ' unused
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = -1                   ' unused
     ObjectArray(ObjectHandle).Text = ""                         ' unused

     SetControlBox% = ObjectHandle

END FUNCTION

FUNCTION SetDirIcon% (ULC%, ULR%, State%, DirType%, DirName$, Fore%, Back%, ObjectArray() AS GUIObject)

     SetDirIcon% = -1                                            ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     X2 = ULC% + 147 - 1
     Y2 = ULR% + 18 - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = DirIconType          ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = Fore%                      ' set Foreground Color
     ObjectArray(ObjectHandle).Back = Back%                      ' set Background Color
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = DirType%          ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = LEN(DirName$)        ' set Text String Len
     ObjectArray(ObjectHandle).Text = DirName$                   ' set Text

     SetDirIcon% = ObjectHandle

END FUNCTION

FUNCTION SetDownArrowBut% (ULC%, ULR%, State%, ObjectArray() AS GUIObject)

     SetDownArrowBut% = -1                                       ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     CALL GetVideoMode(CurrentVideoMode)                         ' set DownArrow button size according to the current video mode
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        X2 = ULC% + 13 - 1
        Y2 = ULR% + 10 - 1
     CASE 18
        X2 = ULC% + 11 - 1
        Y2 = ULR% + 10 - 1
     CASE 91
        X2 = ULC% + 15 - 1
        Y2 = ULR% + 11 - 1
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = DownArrowButType     ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = -1                         ' unused
     ObjectArray(ObjectHandle).Back = -1                         ' unused
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = -1                   ' unused
     ObjectArray(ObjectHandle).Text = ""                         ' unused

     SetDownArrowBut% = ObjectHandle

END FUNCTION

FUNCTION SetExeIcon% (ULC%, ULR%, State%, FileName$, Fore%, Back%, ObjectArray() AS GUIObject)

     SetExeIcon% = -1                                            ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     X2 = ULC% + 147 - 1
     Y2 = ULR% + 18 - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = ExeIconType          ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = Fore%                      ' set Foreground Color
     ObjectArray(ObjectHandle).Back = Back%                      ' set Background Color
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = LEN(FileName$)       ' set Text String Len
     ObjectArray(ObjectHandle).Text = FileName$                  ' set Text

     SetExeIcon% = ObjectHandle

END FUNCTION

FUNCTION SetFileIcon% (ULC%, ULR%, State%, FileName$, Fore%, Back%, ObjectArray() AS GUIObject)

     SetFileIcon% = -1                                           ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     X2 = ULC% + 147 - 1
     Y2 = ULR% + 18 - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = FileIconType         ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = Fore%                      ' set Foreground Color
     ObjectArray(ObjectHandle).Back = Back%                      ' set Background Color
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = LEN(FileName$)       ' set Text String Len
     ObjectArray(ObjectHandle).Text = FileName$                  ' set Text

     SetFileIcon% = ObjectHandle

END FUNCTION

FUNCTION SetFloppyDrive% (ULC%, ULR%, State%, DriveLetter$, ObjectArray() AS GUIObject)

     SetFloppyDrive% = -1                                        ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     X2 = ULC% + 69 - 1                                          ' start sizing the FloppyDrive
     Y2 = ULR% + 27 - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = FloppyDriveType      ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = -1                         ' unused
     ObjectArray(ObjectHandle).Back = -1                         ' unused
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = LEN(DriveLetter$)    ' set Text String Length
     ObjectArray(ObjectHandle).Text = DriveLetter$               ' set Text

     SetFloppyDrive% = ObjectHandle

END FUNCTION

FUNCTION SetFrameBox% (ULC%, ULR%, LRC%, LRR%, FrameTitle$, FrameColor%, FillColor%, ObjectArray() AS GUIObject)

     SetFrameBox% = -1                                           ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     IF (ULC < LRC) THEN
        X1 = ULC
        X2 = LRC
     ELSE
        X1 = LRC
        X2 = ULC
     END IF

     IF (ULR < LRR) THEN
        Y1 = ULR
        Y2 = LRR
     ELSE
        Y1 = LRR
        Y2 = ULR
     END IF

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = X1                          ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = Y1                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = FrameBoxType         ' set Object Type
     ObjectArray(ObjectHandle).State = -1                        ' unused
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = FrameColor%                ' set Foreground Color
     ObjectArray(ObjectHandle).Back = FillColor%                 ' set Background Color
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = LEN(FrameTitle$)     ' set Text String Length
     ObjectArray(ObjectHandle).Text = FrameTitle$                ' set Text

     SetFrameBox% = ObjectHandle

END FUNCTION

FUNCTION SetHardDrive% (ULC%, ULR%, State%, DriveLetter$, ObjectArray() AS GUIObject)

     SetHardDrive% = -1                                          ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     X2 = ULC% + 69 - 1                                          ' start sizing the HardDrive
     Y2 = ULR% + 27 - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = HardDriveType        ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = -1                         ' unused
     ObjectArray(ObjectHandle).Back = -1                         ' unused
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = LEN(DriveLetter$)    ' set Text String Length
     ObjectArray(ObjectHandle).Text = DriveLetter$               ' set Text

     SetHardDrive% = ObjectHandle

END FUNCTION

FUNCTION SetHorizontalScrollBar% (ULC%, ULR%, LRC%, LRR%, State%, BarPosition%, ObjectArray() AS GUIObject)

     SetHorizontalScrollBar% = -1                                ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     IF (ULC < LRC) THEN
        X1 = ULC
        X2 = LRC
     ELSE
        X1 = LRC
        X2 = ULC
     END IF
     Y2 = ULR% + ScrollBarBoxSize - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = X1                          ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = HorizontalScrollBarType     ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = -1                         ' unused
     ObjectArray(ObjectHandle).Back = -1                         ' unused
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = BarPosition%        ' set Bar Position
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = -1                   ' unused
     ObjectArray(ObjectHandle).Text = ""                         ' unused

     SetHorizontalScrollBar% = ObjectHandle

END FUNCTION

FUNCTION SetLeftArrowBut% (ULC%, ULR%, State%, ObjectArray() AS GUIObject)

     SetLeftArrowBut% = -1                                       ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     CALL GetVideoMode(CurrentVideoMode)                         ' set LeftArrow button size according to the current video mode
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        X2 = ULC% + 12 - 1
        Y2 = ULR% + 11 - 1
     CASE 18
        X2 = ULC% + 10 - 1
        Y2 = ULR% + 11 - 1
     CASE 91
        X2 = ULC% + 14 - 1
        Y2 = ULR% + 11 - 1
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = LeftArrowButType     ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = -1                         ' unused
     ObjectArray(ObjectHandle).Back = -1                         ' unused
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = -1                   ' unused
     ObjectArray(ObjectHandle).Text = ""                         ' unused

     SetLeftArrowBut% = ObjectHandle

END FUNCTION

FUNCTION SetMaxBut% (ULC%, ULR%, State%, ObjectArray() AS GUIObject)

     SetMaxBut% = -1                                             ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     SELECT CASE GETCHARSIZE                                     ' set MaxButton size according to the current CHARSIZE
     CASE 0
        ButtonHeight = 16
     CASE 1
        ButtonHeight = 20
     CASE 2
        ButtonHeight = 22
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        ButtonWidth = ButtonHeight + 3
     CASE 18
        ButtonWidth = ButtonHeight + 1
     CASE 91
        ButtonWidth = ButtonHeight + 1
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     X2 = ULC% + ButtonWidth - 1
     Y2 = ULR% + ButtonHeight - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = MaxButType           ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = -1                         ' unused
     ObjectArray(ObjectHandle).Back = -1                         ' unused
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = -1                   ' unused
     ObjectArray(ObjectHandle).Text = ""                         ' unused

     SetMaxBut% = ObjectHandle

END FUNCTION

FUNCTION SetMinBut% (ULC%, ULR%, State%, ObjectArray() AS GUIObject)

     SetMinBut% = -1                                             ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     SELECT CASE GETCHARSIZE                                     ' set MinButton size according to the current CHARSIZE
     CASE 0
        ButtonHeight = 16
     CASE 1
        ButtonHeight = 20
     CASE 2
        ButtonHeight = 22
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        ButtonWidth = ButtonHeight + 3
     CASE 18
        ButtonWidth = ButtonHeight + 1
     CASE 91
        ButtonWidth = ButtonHeight + 1
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     X2 = ULC% + ButtonWidth - 1
     Y2 = ULR% + ButtonHeight - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = MinButType           ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = -1                         ' unused
     ObjectArray(ObjectHandle).Back = -1                         ' unused
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = -1                   ' unused
     ObjectArray(ObjectHandle).Text = ""                         ' unused

     SetMinBut% = ObjectHandle

END FUNCTION

FUNCTION SetMinMaxBut% (ULC%, ULR%, State%, ObjectArray() AS GUIObject)

     SetMinMaxBut% = -1                                          ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     SELECT CASE GETCHARSIZE                                     ' set MinMax button size according to the current CHARSIZE
     CASE 0
        ButtonHeight = 16
     CASE 1
        ButtonHeight = 20
     CASE 2
        ButtonHeight = 22
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        ButtonWidth = ButtonHeight + 3
     CASE 18
        ButtonWidth = ButtonHeight + 1
     CASE 91
        ButtonWidth = ButtonHeight + 1
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     X2 = ULC% + ButtonWidth - 1
     Y2 = ULR% + ButtonHeight - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = MinMaxButType        ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = -1                         ' unused
     ObjectArray(ObjectHandle).Back = -1                         ' unused
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = -1                   ' unused
     ObjectArray(ObjectHandle).Text = ""                         ' unused

     SetMinMaxBut% = ObjectHandle

END FUNCTION

SUB SetObjectEnabled (EnabledFlag%, ObjectHandle%, ObjectArray() AS GUIObject)
'------------------------- SetObjectEnabled -------------------------
'
' SetObjectEnabled will set the Enabled flag for the given Object based
'                  on the passed EnabledFlag.
'
' Calling Convention:
'    SetObjectEnabled(EnabledFlag%, ObjectHandle%, ObjectArray())
'
' EnabledFlag%   As follows:
'                     EnabledFlag = TRUE,  Set Object to Enabled
'                     EnabledFlag = FALSE, Set Object to InEnabled
' ObjectHandle%   Object Handle is the Array Element Number for the
'                 that Object in the ObjectArray().
' ObjectArray()   Array of GUIObject TYPEs.
'
' returned modified       :
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     IF (ObjectHandle% < 1) OR (ObjectHandle% > ObjectArray(0).CurrentUse) THEN         ' check for invalid ObjectHandle%
        EXIT SUB
     END IF

     ObjectArray(ObjectHandle%).Enabled = EnabledFlag%           ' set the flag

END SUB

SUB SetObjectVisible (VisibleFlag%, ObjectHandle%, ObjectArray() AS GUIObject)
'------------------------- SetObjectVisible -------------------------
'
' SetObjectVisible will set the Visible flag for the given Object based
'                  on the passed VisisbleFlag.
'
' Calling Convention:
'    SetObjectVisible(VisibleFlag%, ObjectHandle%, ObjectArray())
'
' VisibleFlag%    As follows:
'                     VisibleFlag = TRUE,  Set Object to Visible
'                     VisibleFlag = FALSE, Set Object to inVisible
' ObjectHandle%   Object Handle is the Array Element Number for the
'                 that Object in the ObjectArray().
' ObjectArray()   Array of GUIObject TYPEs.
'
' returned modified       :
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     IF (ObjectHandle% < 1) OR (ObjectHandle% > ObjectArray(0).CurrentUse) THEN         ' check for invalid ObjectHandle%
        EXIT SUB
     END IF

     ObjectArray(ObjectHandle%).Visible = VisibleFlag%           ' set the flag

END SUB

FUNCTION SetOptionBut% (ULC%, ULR%, State%, Caption$, Fore%, Back%, ObjectArray() AS GUIObject)

     SetOptionBut% = -1                                          ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     SELECT CASE GETCHARSIZE                                     ' set font height according to the current video mode
     CASE 0
        FontHeight = 8
        CharX = 16
     CASE 1
        FontHeight = 14
        CharX = 21
     CASE 2
        FontHeight = 16
        CharX = 23
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     X2 = ULC% + CharX + 8 * LEN(Caption$) + 7 - 1               ' set the OptionButton coordinates
     Y2 = ULR% + FontHeight + 3 - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = OptionButType        ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = Fore%                      ' set Foreground Color
     ObjectArray(ObjectHandle).Back = Back%                      ' set Background Color
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = LEN(Caption$)        ' set Text String Length
     ObjectArray(ObjectHandle).Text = Caption$                   ' set Text

     SetOptionBut% = ObjectHandle

END FUNCTION

FUNCTION SetRightArrowBut% (ULC%, ULR%, State%, ObjectArray() AS GUIObject)

     SetRightArrowBut% = -1                                      ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     CALL GetVideoMode(CurrentVideoMode)                         ' set RightArrow button size according to the current video mode
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        X2 = ULC% + 12 - 1
        Y2 = ULR% + 11 - 1
     CASE 18
        X2 = ULC% + 10 - 1
        Y2 = ULR% + 11 - 1
     CASE 91
        X2 = ULC% + 14 - 1
        Y2 = ULR% + 11 - 1
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = RightArrowButType    ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = -1                         ' unused
     ObjectArray(ObjectHandle).Back = -1                         ' unused
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = -1                   ' unused
     ObjectArray(ObjectHandle).Text = ""                         ' unused

     SetRightArrowBut% = ObjectHandle

END FUNCTION

SUB SetSemiAutoGUI (SemiAutomaticFlag%)
'------------------------ SetSemiAutoGUI ---------------------
'
' See CheckGUIEvent for further explaination.
'
' returned modified       :
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SemiAutoGUIFlag = SemiAutomaticFlag

END SUB

FUNCTION SetTextBox% (ULC%, ULR%, LRC%, LRR%, State%, Justify%, Caption$, HotColor%, Fore%, Back%, ObjectArray() AS GUIObject)

     SetTextBox% = -1                                           ' set default error code
     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     SELECT CASE GETCHARSIZE                                     ' set font height
     CASE 0
        FontHeight = 8
     CASE 1
        FontHeight = 14
     CASE 2
        FontHeight = 16
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     CaptionStringLen = LEN(Caption$)
     IF (CaptionStringLen > 96) THEN EXIT FUNCTION
     CaptionPixelLen = 8 * CaptionStringLen

     X1 = ULC                                                ' set text box coordinates
     IF (LRC = -1) THEN
        X2 = X1 + CaptionPixelLen + 8 - 1
     ELSE
        IF (ULC < LRC) THEN
             X2 = LRC
        ELSE
             X1 = LRC
             X2 = ULC
        END IF
     END IF

     Y1 = ULR
     IF (LRR = -1) THEN
        Y2 = Y1 + FontHeight + 4 - 1
     ELSE
        IF (ULR < LRR) THEN
             Y2 = LRR
        ELSE
             Y1 = LRR
             Y2 = ULR
        END IF
     END IF

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = X1                          ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = Y1                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = TextBoxType          ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = Fore%                      ' set Foreground Color
     ObjectArray(ObjectHandle).Back = Back%                      ' set Background Color
     ObjectArray(ObjectHandle).HotKeyColor = HotColor%           ' set Hot Key Color
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = Justify%        ' set Text Justify
     ObjectArray(ObjectHandle).TextStrLen = LEN(Caption$)        ' set Text String Len
     ObjectArray(ObjectHandle).Text = Caption$                   ' set Text

     SetTextBox% = ObjectHandle

END FUNCTION

FUNCTION SetUpArrowBut% (ULC%, ULR%, State%, ObjectArray() AS GUIObject)

     SetUpArrowBut% = -1                                         ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     CALL GetVideoMode(CurrentVideoMode)                         ' set UpArrow button size according to the current video mode
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        X2 = ULC% + 13 - 1
        Y2 = ULR% + 10 - 1
     CASE 18
        X2 = ULC% + 11 - 1
        Y2 = ULR% + 10 - 1
     CASE 91
        X2 = ULC% + 15 - 1
        Y2 = ULR% + 11 - 1
     CASE ELSE
        EXIT FUNCTION
     END SELECT

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = ULR%                        ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = UpArrowButType       ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = -1                         ' unused
     ObjectArray(ObjectHandle).Back = -1                         ' unused
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = -1                  ' unused
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = -1                   ' unused
     ObjectArray(ObjectHandle).Text = ""                         ' unused

     SetUpArrowBut% = ObjectHandle

END FUNCTION

FUNCTION SetVerticalScrollBar% (ULC%, ULR%, LRC%, LRR%, State%, BarPosition%, ObjectArray() AS GUIObject)

     SetVerticalScrollBar% = -1                                  ' set default error code

     ObjectHandle = GetHandle(ObjectArray())
     IF (ObjectHandle = 0) THEN
        EXIT FUNCTION
     END IF

     IF (ULR < LRR) THEN
        Y1 = ULR
        Y2 = LRR
     ELSE
        Y1 = LRR
        Y2 = ULR
     END IF
     X2 = ULC% + ScrollBarBoxSize - 1

     ObjectArray(ObjectHandle).CurrentUse = -1                   ' set Current Use to USED
     ObjectArray(ObjectHandle).ULC = ULC%                        ' set screen coordinates
     ObjectArray(ObjectHandle).ULR = Y1                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRC = X2                          ' set screen coordinates
     ObjectArray(ObjectHandle).LRR = Y2                          ' set screen coordinates
     ObjectArray(ObjectHandle).ObjectType = VerticalScrollBarType     ' set Object Type
     ObjectArray(ObjectHandle).State = State%                    ' set State
     ObjectArray(ObjectHandle).Visible = TRUE                    ' set Visibility
     ObjectArray(ObjectHandle).Enabled = FALSE                   ' set Enabled
     ObjectArray(ObjectHandle).Fore = -1                         ' unused
     ObjectArray(ObjectHandle).Back = -1                         ' unused
     ObjectArray(ObjectHandle).HotKeyColor = -1                  ' unused
     ObjectArray(ObjectHandle).BarPosition = BarPosition%        ' set Bar Position
     ObjectArray(ObjectHandle).DirectoryType = -1                ' unused
     ObjectArray(ObjectHandle).LongestStringLen = -1             ' unused
     ObjectArray(ObjectHandle).TextJustify = -1                  ' unused
     ObjectArray(ObjectHandle).TextStrLen = -1                   ' unused
     ObjectArray(ObjectHandle).Text = ""                         ' unused

     SetVerticalScrollBar% = ObjectHandle

END FUNCTION

SUB SizeCheckBox (ULC%, ULR%, State%, Caption$, Fore%, Back%, ObjectHeight%, ObjectWidth%)
'-------------------------- SizeCheckBox --------------------------------
'
' SizeCheckBox will size a GUI Check Box.
'
' Calling Convention:
' SizeCheckBox(ULC%, ULR%, State%, Caption$, Fore%, Back%, ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  Caption$           Text for Check Box
'  Fore%              Unused
'  Back%              Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SELECT CASE GETCHARSIZE                                     ' set font height according to CHARSIZE
     CASE 0
        FontHeight = 8
        CharX = 16
     CASE 1
        FontHeight = 14
        CharX = 22
     CASE 2
        FontHeight = 16
        CharX = 24
     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

     ObjectHeight% = FontHeight + 3
     ObjectWidth = CharX + 8 * LEN(Caption$) + 7

END SUB

SUB SizeComboBox (ULC%, ULR%, State%, LongestStringLen%, Caption$, ObjectHeight%, ObjectWidth%)
'------------------------ SizeComboBox ----------------------------------
'
' SizeComboBox will size a GUI Combo Box.
'
' Calling Convention:
' SizeComboBox(ULC%, ULR%, State%, LongestStringLen%, Caption$,_
'              ObjectHeight%, ObjectWidth%)
'
'  ULC%                     Unused
'  ULR%                     Unused
'  State%                   Unused
'  LongestStringLen%        The character length of the longest string
'                           for the item list of the Combo Box.
'  Caption$                 Unused
'  ObjectHeight%            Return Value:  The Height of the Object
'  ObjectWidth%             Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SELECT CASE GETCHARSIZE                                     '  Set ComboBox button size according to the current CHARSIZE
     CASE 0
        ObjectHeight = 16
     CASE 1
        ObjectHeight = 20
     CASE 2
        ObjectHeight = 22
     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        BoxWidth = ObjectHeight + 3
     CASE 18
        BoxWidth = ObjectHeight + 1
     CASE 91
        BoxWidth = ObjectHeight + 1
     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

     ObjectWidth% = 8 * (LongestStringLen% + 2) + (BoxWidth \ 2) + BoxWidth

END SUB

SUB SizeCommandBut (ULC%, ULR%, State%, Caption$, ObjectHeight%, ObjectWidth%)
'------------------------ SizeCommandBut --------------------------------
'
' SizeCommandBut will size a GUI Command Button.
'
' Calling Convention:
' SizeCommandBut(ULC%, ULR%, State%, Caption$, ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  Caption$           Text for the Command Button
'  Fore%              Unused
'  Back%              Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SELECT CASE GETCHARSIZE                                     ' set font height according to CHARSIZE
     CASE 0
        FontHeight = 8
     CASE 1
        FontHeight = 14
     CASE 2
        FontHeight = 16
     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

     ObjectHeight% = 18 + FontHeight
     ObjectWidth = 8 * LEN(Caption$) + 40

END SUB

SUB SizeControlBox (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'------------------------ SizeControlBox --------------------------------
'
' SizeControlBox will size a GUI Control Box.
'
' Calling Convention:
' SizeControlBox(ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SELECT CASE GETCHARSIZE                                     ' set ControlBox size according to the current CHARSIZE
     CASE 0
        ObjectHeight% = 16
     CASE 1
        ObjectHeight% = 20
     CASE 2
        ObjectHeight% = 22
     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        ObjectWidth% = ObjectHeight% + 3
     CASE 18
        ObjectWidth% = ObjectHeight% + 1
     CASE 91
        ObjectWidth% = ObjectHeight% + 1
     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

END SUB

SUB SizeDirIcon (ULC%, ULR%, State%, DirType%, DirName$, Fore%, Back%, ObjectHeight%, ObjectWidth%)
'------------------------ SizeDirIcon ----------------------------------
'
' SizeDirIcon will size a GUI File Manager Dir Icon.
'
' Calling Convention:
' SizeDirIcon(ULC%, ULR%, State%, DirType%, DirName$, Fore%, Back%,_
'             ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  DirType%           Unused
'  DirName$           Unused
'  Fore%              Unused
'  Back%              Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     ObjectHeight% = 18
     ObjectWidth% = 147

END SUB

SUB SizeDownArrowBut (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'----------------------- SizeDownArrowBut ------------------------------
'
' SizeDownArrowBut will size a GUI Down Arrow Button.
'
' Calling Convention:
' SizeDownArrowBut(ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     CALL GetVideoMode(CurrentVideoMode)                         ' set DownArrow button size according to the current video mode
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        ObjectHeight% = 10
        ObjectWidth% = 13

     CASE 18
        ObjectHeight% = 10
        ObjectWidth% = 11

     CASE 91
        ObjectHeight% = 11
        ObjectWidth% = 15

     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

END SUB

SUB SizeExeIcon (ULC%, ULR%, State%, FileName$, Fore%, Back%, ObjectHeight%, ObjectWidth%)
'------------------------ SizeExeIcon ----------------------------------
'
' SizeExeIcon will size a GUI File Manager Executeable Icon.
'
' Calling Convention:
' SizeExeIcon(ULC%, ULR%, State%, FileName$, Fore%, Back%,_
'             ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  FileName$          Unused
'  Fore%              Unused
'  Back%              Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     ObjectHeight% = 18
     ObjectWidth% = 147

END SUB

SUB SizeFileIcon (ULC%, ULR%, State%, FileName$, Fore%, Back%, ObjectHeight%, ObjectWidth%)
'------------------------ SizeFileIcon ----------------------------------
'
' SizeFileIcon will size a GUI File Manager File Icon.
'
' Calling Convention:
' SizeFileIcon(ULC%, ULR%, State%, FileName$, Fore%, Back%_
'              ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  FileName$          Unused
'  Fore%              Unused
'  Back%              Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     ObjectHeight% = 18
     ObjectWidth% = 147

END SUB

SUB SizeFloppyDrive (ULC%, ULR%, State%, DriveLetter$, ObjectHeight%, ObjectWidth%)
'----------------------- SizeFloppyDrive --------------------------------
'
' SizeFloppyDrive will size a GUI Floppy Drive.
'
' Calling Convention:
' SizeFloppyDrive(ULC%, ULR%, State%, DriveLetter$, ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  DriveLetter$       Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     ObjectHeight% = 27
     ObjectWidth% = 69

END SUB

SUB SizeHardDrive (ULC%, ULR%, State%, DriveLetter$, ObjectHeight%, ObjectWidth%)
'------------------------- SizeHardDrive --------------------------------
'
' SizeHardDrive will size a GUI Hard Drive.
'
' Calling Convention:
' SizeHardDrive(ULC%, ULR%, State%, DriveLetter$, ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  DriveLetter$       Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     ObjectHeight% = 27
     ObjectWidth% = 69

END SUB

SUB SizeHorizontalScrollBar (ULC%, ULR%, LRC%, LRR%, State%, BarPosition%, ObjectHeight%, ObjectWidth%)
'--------------------- SizeHorizontalScrollBar --------------------------
'
' SizeHorizontalScrollBar will size a GUI Horizontal Scroll
'                         Bar.
'
' Calling Convention:
' SizeHorizontalScrollBar(ULC%, ULR%, LRC%, LRR%, State%, BarPosition%,_
'                         ObjectHeight%, ObjectWidth%)
'
'  ULC%          Upper left corner X coordinate (0 <= ULC <= YMAX)
'  ULR%          Unused
'  LRC%          Bottom right corner X coordinate.  This parameter will
'                set the Horizontal Scroll Bar Width.
'  LRR%          Unused
'  State%        Unused
'  BarPosition%  Unused
'  ObjectHeight% Return Value:  The Height of the Object
'  ObjectWidth%  Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     IF (ULC < LRC) THEN
        X1 = ULC
        X2 = LRC
     ELSE
        X1 = LRC
        X2 = ULC
     END IF

     ObjectHeight% = ScrollBarBoxSize
     ObjectWidth% = X2 - X1 + 1

END SUB

SUB SizeLeftArrowBut (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'----------------------- SizeLeftArrowBut ------------------------------
'
' SizeLeftArrowBut will size a GUI Left Arrow Button.
'
' Calling Convention:
' SizeLeftArrowBut(ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     CALL GetVideoMode(CurrentVideoMode)                         ' set LeftArrow button size according to the current video mode
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        ObjectHeight% = 11
        ObjectWidth% = 12

     CASE 18
        ObjectHeight% = 11
        ObjectWidth% = 10

     CASE 91
        ObjectHeight% = 11
        ObjectWidth% = 14

     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

END SUB

SUB SizeMaxBut (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'-------------------------- SizeMaxBut ----------------------------------
'
' SizeMaxBut will size a GUI Maximum Button at the passed
'            coordinates with the passed State.
'
' Calling Convention:
' SizeMaxBut(ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SELECT CASE GETCHARSIZE                                     ' set MaxButton size according to the current CHARSIZE
     CASE 0
        ObjectHeight% = 16

     CASE 1
        ObjectHeight% = 20

     CASE 2
        ObjectHeight% = 22

     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        ObjectWidth% = ObjectHeight% + 3
     CASE 18
        ObjectWidth% = ObjectHeight% + 1
     CASE 91
        ObjectWidth% = ObjectHeight% + 1
     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

END SUB

SUB SizeMinBut (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'-------------------------- SizeMinBut ----------------------------------
'
' SizeMinBut will size a GUI Minimum Button.
'
' Calling Convention:
' SizeMinBut(ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SELECT CASE GETCHARSIZE                                     ' set MinButton size according to the current CHARSIZE
     CASE 0
        ObjectHeight% = 16

     CASE 1
        ObjectHeight% = 20

     CASE 2
        ObjectHeight% = 22

     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        ObjectWidth% = ObjectHeight% + 3
     CASE 18
        ObjectWidth% = ObjectHeight% + 1
     CASE 91
        ObjectWidth% = ObjectHeight% + 1
     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

END SUB

SUB SizeMinMaxBut (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'----------------------- SizeMinMaxBut ----------------------------------
'
' SizeMinMaxBut will size a GUI Minimum/Maximum Button.
'
' Calling Convention:
' SizeMinMaxBut(ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SELECT CASE GETCHARSIZE                                     ' set MinMax button size according to the current CHARSIZE
     CASE 0
        ObjectHeight% = 16

     CASE 1
        ObjectHeight% = 20

     CASE 2
        ObjectHeight% = 22

     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

     CALL GetVideoMode(CurrentVideoMode)
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        ObjectWidth% = ObjectHeight% + 3
     CASE 18
        ObjectWidth% = ObjectHeight% + 1
     CASE 91
        ObjectWidth% = ObjectHeight% + 1
     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

END SUB

SUB SizeOptionBut (ULC%, ULR%, State%, Caption$, Fore%, Back%, ObjectHeight%, ObjectWidth%)
'-------------------------- SizeOptionBut --------------------------------
'
' SizeOptionBut will size a GUI Option Button (Radio Button).
'
' Calling Convention:
' SizeOptionBut(ULC%, ULR%, State%, Caption$, Fore%, Back%, ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  Caption$           Text for Option Button
'  Fore%              Unused
'  Back%              Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SELECT CASE GETCHARSIZE                                     ' set font height according to CHARSIZE
     CASE 0
        FontHeight = 8
        CharX = 16
     CASE 1
        FontHeight = 14
        CharX = 21
     CASE 2
        FontHeight = 16
        CharX = 23
     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

     ObjectHeight% = FontHeight + 3
     ObjectWidth = CharX + 8 * LEN(Caption$) + 7

END SUB

SUB SizeRightArrowBut (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'---------------------- SizeRightArrowBut ------------------------------
'
' SizeRightArrowBut will size a GUI Right Arrow Button.
'
' Calling Convention:  SizeRightArrowBut(ULC%, ULR%, State%,_
'                                        ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     CALL GetVideoMode(CurrentVideoMode)                         '  set RightArrow button size according to the current video mode
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        ObjectHeight% = 11
        ObjectWidth% = 12

     CASE 18
        ObjectHeight% = 11
        ObjectWidth% = 10

     CASE 91
        ObjectHeight% = 11
        ObjectWidth% = 14

     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

END SUB

SUB SizeTextBox (ULC%, ULR%, LRC%, LRR%, State%, Justify%, Caption$, HotColor%, Fore%, Back%, ObjectHeight%, ObjectWidth%)
'------------------------ SizeTextBox ----------------------------------
'
' SizeTextBox will size a GUI Text Box at the passed
'             coordinates.
'
' Calling Convention:
' SizeTextBox(ULC%, ULR%, LRC%, LRR%, State%, Justify%, Caption$, HotColor%, Fore%, Back%_
'             ObjectHeight%, ObjectWidth%)
'
'  ULC%    Upper left corner X coordinate (0 <= ULC <= XMAX)
'  ULR%    Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  LRC%    Bottom right corner X coordinate (0 <= LRC <= XMAX).
'              If LRC = -1, then SizeText will calculate the
'                               appropriate coordinate based on the
'                               string length.
'  LRR%    Bottom right corner Y coordinate (0 <= LRR <= YMAX).
'              If LRR = -1, then SizeText will calculate the
'                               appropriate coordinate based on the
'                               Font Height.
'  State%             Unused
'  Justify%           Unused
'  Caption$           Text to be placed in the Text Box
'  HotColor%          Unused
'  Fore%              Unused
'  Back%              Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     SELECT CASE GETCHARSIZE                                     '  set font height
     CASE 0
        FontHeight = 8
     CASE 1
        FontHeight = 14
     CASE 2
        FontHeight = 16
     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

     CaptionStringLen = LEN(Caption$)
     CaptionPixelLen = 8 * CaptionStringLen

     X1 = ULC                                                    ' set TextBox coordinates
     IF (LRC = -1) THEN
        X2 = X1 + CaptionPixelLen + 8 - 1
     ELSE
        IF (ULC < LRC) THEN
        X2 = LRC
        ELSE
        X1 = LRC
        X2 = ULC
        END IF
     END IF

     Y1 = ULR
     IF (LRR = -1) THEN
        Y2 = Y1 + FontHeight + 4 - 1
     ELSE
        IF (ULR < LRR) THEN
        Y2 = LRR
        ELSE
        Y1 = LRR
        Y2 = ULR
        END IF
     END IF

     ObjectHeight% = Y2 - Y1 + 1
     ObjectWidth% = X2 - X1 + 1

END SUB

SUB SizeUpArrowBut (ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'------------------------- SizeUpArrowBut ------------------------------
'
' SizeUpArrowBut will size a GUI Up Arrow Button.
'
' Calling Convention:
' SizeUpArrowBut(ULC%, ULR%, State%, ObjectHeight%, ObjectWidth%)
'
'  ULC%               Unused
'  ULR%               Unused
'  State%             Unused
'  ObjectHeight%      Return Value:  The Height of the Object
'  ObjectWidth%       Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     CALL GetVideoMode(CurrentVideoMode)                         ' set UpArrow button size according to the current video mode
     SELECT CASE CurrentVideoMode
     CASE 14, 16
        ObjectHeight% = 10
        ObjectWidth% = 13

     CASE 18
        ObjectHeight% = 10
        ObjectWidth% = 11

     CASE 91
        ObjectHeight% = 11
        ObjectWidth% = 15

     CASE ELSE
        ObjectHeight% = 0
        ObjectWidth% = 0
        EXIT SUB
     END SELECT

END SUB

SUB SizeVerticalScrollBar (ULC%, ULR%, LRC%, LRR%, State%, BarPosition%, ObjectHeight%, ObjectWidth%)
'------------------ SizeVerticalScrollBar ------------------------------
'
' SizeVerticalScrollBar will size a GUI Vertical Scroll Bar.
'
' Calling Convention:
' SizeVerticalScrollBar(ULC%, ULR%, LRC%, LRR%, State%, BarPosition%,_
'                       ObjectHeight%, ObjectWidth%)
'
'  ULC%          Unused
'  ULR%          Upper left corner Y coordinate (0 <= ULR <= YMAX)
'  LRC%          Unused
'  LRR%          Bottom right corner Y coordinate.  This parameter will
'                set the Vertical Scroll Bar Height.
'  State%        Unused
'  BarPosition%  Unused
'  ObjectHeight% Return Value:  The Height of the Object
'  ObjectWidth%  Return Value:  The Width of the Object
'
'  returned modified       : ObjectHeight%, ObjectWidth%
'  compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     IF (ULR < LRR) THEN
        Y1 = ULR
        Y2 = LRR
     ELSE
        Y1 = LRR
        Y2 = ULR
     END IF

     ObjectHeight% = Y2 - Y1 + 1
     ObjectWidth% = ScrollBarBoxSize

END SUB

SUB UpdateGUI (ObjectArray() AS GUIObject)
'----------------------- UpdateGUI --------------------------------
'
' UpdateGUI will draw/redraw every object in the object array.
'
' Calling Convention:
'    UpdateGUI(ObjectArray())
'
' ObjectArray()   Array of GUIObject TYPEs.
'
' returned modified       :
' compiler required       : QuickBASIC 4.0 or higher
'
'--- end comments


     NumberOfObjects = ObjectArray(0).CurrentUse

     '--- animate FrameBoxes first
     FOR Index = 1 TO NumberOfObjects
        IF (ObjectArray(Index).ObjectType = FrameBoxType) THEN
           TempState = ObjectArray(Index).State                  ' save actual state
           IF (TempState <> INVALIDSTATE) THEN
                ObjectArray(Index).State = INVALIDSTATE          ' set invalid state
           ELSE
                ObjectArray(Index).State = NORMAL                ' set different from current state
           END IF

           CALL AnimateObject(Index, TempState, ObjectArray())   ' redraw object
        END IF
     NEXT Index

     '--- animate every other object
     FOR Index = 1 TO NumberOfObjects
        TempObjectType = ObjectArray(Index).ObjectType
        IF (TempObjectType <> FrameBoxType) THEN
           TempState = ObjectArray(Index).State                  ' save actual state
           IF (TempState <> INVALIDSTATE) THEN
                ObjectArray(Index).State = INVALIDSTATE          ' set invalid state
           ELSE
                ObjectArray(Index).State = NORMAL                ' set different from current state
           END IF

           IF (TempObjectType = HorizontalScrollBarType) OR (TempObjectType = VerticalScrollBarType) THEN
                BarPosition = ObjectArray(Index).BarPosition
                CALL AnimateScrollBarObject(Index, TempState, BarPosition, ObjectArray())  ' redraw scroll bar
           ELSE
                CALL AnimateObject(Index, TempState, ObjectArray())   ' redraw object
           END IF
        END IF
     NEXT Index

END SUB


